################################################################################################################################
#Configure_Circle
################################################################################################################################
import inspect

class Configure_Group:
    def __init__(self, Global, Configure):
        try:
            self.Global = Global
            self.Configure = Configure
            self.Widget = []
            self.ID = False
            self.Root = False
            self.Root_ID = False
            self.Element = False
            
            Top = 0
            
            #Frame
            Fixture = self.Configure.Frame.Locate(100, 95, 0, 5)
            self.Frame = self.Global['Gluonix'].Scroll(self.Configure.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=False)
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            self.Configure.Widget.append(self)
            
            #Name Label
            Top += 2
            Fixture = self.Frame.Locate(25, 5, 3, Top)
            self.Name_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Name_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Name_Label.Config(Foreground='#000000', Value="Name:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Name_Label.Create()
            
            #Name Entry
            Fixture = self.Frame.Locate(60, 5, 28, Top)
            self.Name_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Name_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Name_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='left', Border_Size=1)
            self.Name_Entry.Bind(On_Key_Release=lambda E: self.Update_Name())
            self.Name_Entry.Create()
            
            #Visibilty
            Fixture = self.Frame.Locate(7, 5, 90, Top)
            self.Visibilty_Image = {True: 'Visibilty_On', False: 'Visibilty_Off'}
            self.Visibilty = self.Global['Gluonix'].Image(self.Frame)
            self.Visibilty.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Visibilty.Config(Border_Size=0, Path=self.Global['Image'](self.Visibilty_Image[True]))
            self.Visibilty.Bind(On_Click=lambda E: self.Update_Visibilty())
            self.Visibilty.Create()
            
            #Move Label
            Top += 7
            Fixture = self.Frame.Locate(25, 5, 3, Top)
            self.Move_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Move_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Move_Label.Config(Foreground='#000000', Value="Relative Move", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Move_Label.Create()
            
            #Left Label
            Top += 7
            Fixture = self.Frame.Locate(25, 5, 3, Top)
            self.Left_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Left_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Left_Label.Config(Foreground='#000000', Value="Left:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Left_Label.Create()
            
            #Left Entry
            Fixture = self.Frame.Locate(40, 5, 28, Top)
            self.Left_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Left_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Left_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Left_Entry.Bind(On_Key_Release=lambda E: self.Check_Left())
            self.Left_Entry.Create()
            
            #Top Label
            Top += 7
            Fixture = self.Frame.Locate(25, 5, 3, Top)
            self.Top_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Top_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Top_Label.Config(Foreground='#000000', Value="Top:", Font_Size=10, Font_Weight='normal', Align='w', Border_Size=0)
            self.Top_Label.Create()
            
            #Top Entry
            Fixture = self.Frame.Locate(40, 5, 28, Top)
            self.Top_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Top_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Top_Entry.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=9, Font_Weight='normal', Align='center', Border_Size=1)
            self.Top_Entry.Bind(On_Key_Release=lambda E: self.Check_Top())
            self.Top_Entry.Create()
            
            #Move Button
            Top += 7
            Fixture = self.Frame.Locate(40, 5, 28, Top)
            self.Move_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Move_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3], Resize=False, Move=False)
            self.Move_Button.Config(Background='#e1e1e1', Foreground='black', Value='Move', Font_Size=12, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Move_Button.Bind(On_Hover_In=lambda E: self.Move_Button.Config(Border_Color='#0078d7', Background='#d5dcf0'))
            self.Move_Button.Bind(On_Hover_Out=lambda E: self.Move_Button.Config(Border_Color='#adadad', Background='#e1e1e1'))
            self.Move_Button.Bind(On_Click=lambda E: self.Move())
            self.Move_Button.Create()
            
            #Update Scroll
            self.Frame.Update(self.Visibilty)
            
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self, ID=False):
        try:
            if ID!=self.ID:
                self.Root = False
                self.Root_ID = False
                self.Element = False
                self.ID = ID
                Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{self.ID}'", Keys=True)
                if len(Widget_Data)==1:
                    Widget_Data = Widget_Data[0]
                    self.Root_ID = Widget_Data['Root']
                    self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                    self.Name_Entry.Set(Widget_Data['Name'])
                    self.Left_Entry.Set(0)
                    self.Top_Entry.Set(0)
                    self.Configure.Hide_All()
                    self.Configure.Current = self
                    self.Frame.Show()
                    Temp_Root = Widget_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!='Root':
                        Widget_Data = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Widget_Data)==1:
                            Widget_Data = Widget_Data[0]
                            Temp_Root = Widget_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    Root_Split = Root.split('.')
                    Current = ''
                    for Each in Root_Split:
                        if Current:
                            Current = Current+'.'+Each
                        else:
                            Current = Each
                        if Current!='Root':
                            Temp_Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Current)
                            Temp_Root.Show()
                    self.Root = self.Global['Custom'].Get_Attr_Class(self.Configure.Design, Root)
                    self.Element = getattr(self.Root, self.ID)
                    self.Visibilty.Set(self.Global['Image'](self.Visibilty_Image[self.Element._Display]))
                    if self.Element._Display:
                        self.Element.Show()
                else:
                    self.Global['Message'].Show('Error', 'Project Files Corrupted\nReopen Project')
            else:
                self.Element.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()
            
    def Movement_Update(self):
        try:
            pass
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Name(self):
        try:
            Name = self.Name_Entry.Get()
            TempName = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE (`ID`!='{self.ID}' AND `Root`='{self.Root_ID}' AND `Name`='{Name}')")
            if self.Global['Custom'].Valid_Variable(Name) and len(TempName)==0:
                self.Configure.Design.Database.Post(f"UPDATE `Frame` SET `Name`='{Name}' WHERE `ID`='{self.ID}'")
                self.Configure.Design.Element.Tree.Edit(Name=f'  {Name}')
                self.Name_Entry.Config(Border_Color='#000000', Border_Size=1)
            else:
                self.Name_Entry.Config(Border_Color='red', Border_Size=1)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Visibilty(self):
        try:
            if self.Element:
                if self.Element._Display:
                    self.Element.Hide()
                else:
                    self.Element.Show()
                self.Visibilty.Set(self.Global['Image'](self.Visibilty_Image[self.Element._Display]))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Check_Left(self):
        try:
            Left = self.Left_Entry.Get()
            if self.Global['Custom'].Is_Float(Left):
                self.Left_Entry.Config(Border_Color='#000000', Border_Size=1)
                return True
            else:
                self.Left_Entry.Config(Border_Color='red', Border_Size=1)
                return False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Check_Top(self):
        try:
            Top = self.Top_Entry.Get()
            if self.Global['Custom'].Is_Float(Top):
                self.Top_Entry.Config(Border_Color='#000000', Border_Size=1)
                return True
            else:
                self.Top_Entry.Config(Border_Color='red', Border_Size=1)
                return False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Update_Move(self, ID, Left, Top):
        try:
            Widgets = self.Configure.Design.Database.Get(f"SELECT * FROM `Item` WHERE `Root`='{ID}'", Keys=True)
            for Widget in Widgets:
                self.Configure.Design.Database.Post(f"UPDATE `Item` SET Left={Widget['Left']+Left}, Top={Widget['Top']+Top} WHERE `ID`='{Widget['ID']}'")
            Widgets = self.Configure.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `Root`='{ID}'", Keys=True)
            for Widget in Widgets:
                self.Configure.Design.Database.Post(f"UPDATE `Widget` SET Left={Widget['Left']+Left}, Top={Widget['Top']+Top} WHERE `ID`='{Widget['ID']}'")
            Widgets = self.Configure.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `Root`='{ID}'", Keys=True)
            for Widget in Widgets:
                self.Configure.Design.Database.Post(f"UPDATE `Frame` SET Left={Widget['Left']+Left}, Top={Widget['Top']+Top} WHERE `ID`='{Widget['ID']}'")
                if Widget['Type']=='Group':
                    self.Update_Move(Widget['ID'], Left, Top)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Move(self):
        try:
            if self.Element:
                if self.Check_Left() and self.Check_Top():
                    Left = float(self.Left_Entry.Get())
                    Top = float(self.Top_Entry.Get())
                    self.Element.Move(Left, Top)
                    self.Left_Entry.Set(0)
                    self.Top_Entry.Set(0)
                    self.Update_Move(self.ID, Left, Top)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


# IMPORT LIBRARIES
import threading, math, time
from .N_Custom import Event_Bind_Canvas

class Canvas_Polygon:
    def __init__(self, Main):
        self._Canvas = Main
        self._Main = Main
        self._Config = ['Name', 'Outline', 'Fill', 'Thickness', 'Resize', 'Translucent', 'Alpha']
        self._Display = True
        self._Resize = True
        self._Name = False
        self._Last_Name = False
        self._Type = 'Canvas_Polygon'
        self._Points = []
        self._Points_Current = []
        self._Outline = '#000000'
        self._Fill = '#000000'
        self._Thickness = 1
        self._Translucent = False
        self._Alpha = 25
        Points = [0, 0, 0, 0]
        self._Widget = self._Canvas._Frame.create_polygon(Points, outline=self._Outline, width=self._Thickness, fill=self._Fill)
        self._Canvas._Widget.append(self)
        self._Canvas._Item.append(self)
        self._On_Show = False
        self._On_Hide = False

    def __str__(self):
        return "Nucleon_Glunoix_Canvas_Polygon[]"

    def __repr__(self):
        return "Nucleon_Glunoix_Canvas_Polygon[]"

    def Copy(self, Name=False, Main=False):
        try:
            if not Main:
                Main = self._Canvas
            if Main._Type in ['Canvas', 'Scroll', 'Group']:
                Temp_Main = Main
                Temp_Type = Temp_Main._Type
                while Temp_Type=='Group':
                    Temp_Main = Temp_Main._Main
                    Temp_Type = Temp_Main._Type
                if Temp_Type=='Canvas' or Temp_Type=='Scroll':
                    Instance = type(self)(Main)
                    for Key in self._Config:
                        if hasattr(self, "_"+Key):
                            setattr(Instance, "_"+Key, getattr(self, "_"+Key))
                    if Name:
                        setattr(Instance, "_Name", Name)
                    Instance.Create()
                    return Instance
                else:
                    raise Exception('Widget can only copy to Canvas/Scroll')
            else:
                raise Exception('Widget can only copy to Canvas/Scroll')
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Copy -> {E}")

    def Hide(self):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='hidden')
            self._Display = False
            if self._On_Hide:
                self._On_Hide()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type}-> Hide -> {E}")
            
    def Show(self):
        try:
            if not self._Display and self._Resize and self._Canvas._Type!='Scroll':
                self.Create()
            self._Canvas._Frame.itemconfigure(self._Widget, state='normal')
            self._Canvas._Frame.tag_raise(self._Widget)
            self._Display = True
            if self._On_Show:
                self._On_Show()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Show -> {E}")
    
    def Delete(self):
        try:
            self._Canvas._Widget.remove(self)
            self._Canvas._Frame.delete(self._Widget)
            if self:
                del self
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Delete -> {E}")
        
    def Bind(self, **Input):
        try:
            if 'On_Show' in Input:
                self._On_Show = Input['On_Show']
            if 'On_Hide' in Input:
                self._On_Hide = Input['On_Hide']
            Event_Bind_Canvas(self._Canvas._Frame, self._Widget, **Input)
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Bind -> {E}")
            
    def Config_Get(self, *Input):
        try:
            Return = {}
            for Each in self._Config:
                if Each in Input:
                    Return[Each] = getattr(self, "_"+Each)
            return Return
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config_Get -> {E}")
                
    def Config(self, **Input):
        try:
            Run = False
            for Each in self._Config:
                if Each in Input:
                    Value = Input[Each]
                    setattr(self, "_"+Each, Value)
                    setattr(self, "_"+Each+"_Current", Value)
                    Run = True
            if Run:
                self.Create()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Config -> {E}")
            
    def Move(self, Left=None, Top=None):
        try:
            if Left is None and Top is None:
                return False
            Left = 0 if Left is None else Left
            Top = 0 if Top is None else Top
            self._Points = [[X + Left, Y + Top] for X, Y in self._Points]
            if Left != 0 or Top != 0:
                self.Create()
            return True
        except Exception as Error:
            self._Canvas._GUI.Error(f"{self._Type} -> Move -> {Error}")
            
    def Center(self, Left=None, Top=None):
        try:
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            X = (X1+X2)/2
            Y = (Y1+Y2)/2
            return [X, Y]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Center -> {E}")
        
    def Position(self, Left=None, Top=None):
        try:
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            return [X1, Y1]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Position -> {E}")
            
    def Size(self, Width=False, Height=False):
        try:
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            Width = X2 - X1
            Height = Y2 - Y1
            return [Width, Height]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Size -> {E}")
        
    def Ratio(self):
        try:
            return [1, 1]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Ratio -> {E}")
    
    def Enlarge(self, Value=None):
        try:
            return True
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Enlarge -> {E}")
    
    def Shrink(self, Value=None):
        try:
            return True
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Shrink -> {E}")
            
    def Box(self):
        try:
            Box = self._Canvas._Frame.bbox(self._Widget)
            X1, Y1, X2, Y2 = Box
            Width = X2 - X1
            Height = Y2 - Y1
            return [X1, Y1, Width, Height]
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Box -> {E}")

    def Add(self, X, Y):
        try:
            self._Points.append([X, Y])
            self._Points_Current.append([X, Y])
            self.Create()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Add -> {E}")

    def Remove(self, Index):
        try:
            if 0 <= Index < len(self._Points):
                self._Points.pop(Index)
                self._Points_Current.pop(Index)
                self.Create()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Remove -> {E}")

    def Replace(self, Points):
        try:
            self._Points = Points
            self._Points_Current = Points
            self.Create()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Replace -> {E}")
            
    def Stripple(self):
        try:
            if 0 <= self._Alpha <= 12:
                return 12
            elif 13 <= self._Alpha <= 25:
                return 25
            elif 26 <= self._Alpha <= 50:
                return 50
            elif 51 <= self._Alpha <= 75:
                return 75
            else:
                return 100
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Stripple -> {E}")

    def Create(self):
        try:
            if self._Resize and self._Canvas._Type!='Scroll':
                Left, Right, Width, Height = self._Canvas.Box()
                Width_Ratio = Width/self._Canvas._Width
                Height_Ratio = Height/self._Canvas._Height
            else:
                Width_Ratio = 1
                Height_Ratio = 1
            Stripple = f'gray{self.Stripple()}' if self._Translucent else ''
            self._Canvas._Frame.itemconfig(self._Widget, outline=self._Outline, width=self._Thickness, fill=self._Fill, stipple=Stripple)
            if len(self._Points)>1:
                Points = [[X * Width_Ratio, Y * Height_Ratio] for X, Y in self._Points]
                Points = [Item for Pair in Points for Item in Pair]
            else:
                Points = [0, 0, 0, 0]
            self._Canvas._Frame.coords(self._Widget, Points)
            if self._Name!=self._Last_Name:
                if self._Last_Name:
                    if self._Last_Name in self._Canvas.__dict__:
                        del self._Canvas.__dict__[self._Last_Name]
                if self._Name:
                    self._Canvas.__dict__[self._Name] = self
                self._Last_Name = self._Name
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Create -> {E}")
            
    def Resize(self, Event):
        try:
            self._Canvas._Frame.itemconfigure(self._Widget, state='normal')
            self._Canvas._Frame.tag_raise(self._Widget)
            if self._Display:
                self.Create()
        except Exception as E:
            self._Canvas._GUI.Error(f"{self._Type} -> Resize -> {E}")


################################################################################################################################
#Design
################################################################################################################################
#Default Libraries
import inspect
import os
import shutil
import random
import string
import time

#Program
from .Compare.N_Element import Element

class Compare:
    def __init__(self, Global, Main):
        try:
            self.Global = Global
            self.Main = Main
            self.Widget = []
            self.Database_1 = False
            self.Database_2 = False
            self.Project_1 = False
            self.Project_2 = False
            self.Project_1_Path = False
            self.Project_2_Path = False
            self.From = False
            self.To = False
            
            #Frame
            Fixture = self.Main.Frame.Locate(100, 100, 0, 0)
            self.Frame = self.Global['Gluonix'].Frame(self.Main.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Border_Size=0, Display=False)
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            self.Main.Widget.append(self.Frame)
            
            #Label 1
            Fixture = self.Frame.Locate(40, 10, 0, 0)
            self.Label_1 = self.Global['Gluonix'].Label(self.Frame)
            self.Label_1.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_1.Config(Border_Size=0, Display=True)
            self.Label_1.Config(Foreground='black', Font_Size=16, Font_Weight='bold', Align='center')
            self.Label_1.Config(Resize=True, Move=True)
            self.Label_1.Create()
            
            #Display 1 Select
            Fixture = self.Frame.Locate(8, 4, 16, 7)
            self.Display_1_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Display_1_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_1_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=11, Font_Weight='normal', Align='center', Border_Size=1)
            self.Display_1_Select.Config(Hover_Background='#EEEEEE', Hover_Border_Color='#0078d7')
            self.Display_1_Select.Bind(On_Change=lambda E: self.Update(Partial=True))
            self.Display_1_Select.Create()
            
            #Label 2
            Fixture = self.Frame.Locate(40, 10, 60, 0)
            self.Label_2 = self.Global['Gluonix'].Label(self.Frame)
            self.Label_2.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label_2.Config(Border_Size=0, Display=True)
            self.Label_2.Config(Foreground='black', Font_Size=16, Font_Weight='bold', Align='center')
            self.Label_2.Config(Resize=True, Move=True)
            self.Label_2.Create()
            
            #Display 2 Select
            Fixture = self.Frame.Locate(8, 4, 76, 7)
            self.Display_2_Select = self.Global['Gluonix'].Select(self.Frame)
            self.Display_2_Select.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Display_2_Select.Config(Background='#FFFFFF', Foreground='#000000', Font_Size=11, Font_Weight='normal', Align='center', Border_Size=1)
            self.Display_2_Select.Config(Hover_Background='#EEEEEE', Hover_Border_Color='#0078d7')
            self.Display_2_Select.Bind(On_Change=lambda E: self.Update(Partial=True))
            self.Display_2_Select.Create()
            
            #Copy 1->2
            Fixture = self.Frame.Locate(12, 6, 44, 30)
            self.Copy12 = self.Global['Gluonix'].Compound(self.Frame)
            self.Copy12.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Copy12.Config(Background='#e1e1e1', Foreground='black', Value='COPY       ', Font_Size=10, Font_Weight='bold', Border_Size=1, Border_Color='#adadad')
            self.Copy12.Config(Compound='right', Path=self.Global['Image']('Arrow_Right'))
            self.Copy12.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Copy12.Bind(On_Click=lambda E: self.Copy_From_1_To_2())
            self.Copy12.Create()
            
            #Copy 1->2
            Fixture = self.Frame.Locate(12, 6, 44, 45)
            self.Copy21 = self.Global['Gluonix'].Compound(self.Frame)
            self.Copy21.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Copy21.Config(Background='#e1e1e1', Foreground='black', Value='       COPY', Font_Size=10, Font_Weight='bold', Border_Size=1, Border_Color='#adadad')
            self.Copy21.Config(Compound='left', Path=self.Global['Image']('Arrow_Left'))
            self.Copy21.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Copy21.Bind(On_Click=lambda E: self.Copy_From_2_To_1())
            self.Copy21.Create()
            
                #Close
            Fixture = self.Frame.Locate(12, 6, 44, 2)
            self.Close = self.Global['Gluonix'].Label(self.Frame)
            self.Close.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Close.Config(Background='#e1e1e1', Foreground='black', Value='Close', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Close.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Close.Bind(On_Click=lambda E: self.Close_Compare())
            self.Close.Create()
            
            #Display View
            self.Element_1 = Element(self.Global, self)
            self.Element_2 = Element(self.Global, self)
            Fixture = self.Frame.Locate(0, 0, 60, 0)
            self.Element_2.Frame.Config(Left=Fixture[2])
                    
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
    
    def Copy_From_1_To_2(self):
        try:
            self.Global['Message'].Hide()
            From = self.Element_1.Tree.Current()
            if From:
                Element_ID = self.Element_1.Tree.Get(From)
                if Element_ID[0]!=self.Element_1.Parent:
                    self.From = [Element_ID, 1, From]
                    To = self.Element_2.Tree.Current()
                    if To:
                        Element_ID = self.Element_2.Tree.Get(To)
                        if Element_ID[0]!=self.Element_2.Parent:
                            self.To = [Element_ID, 2, To]
                            self.Copy()
                            return
                    self.Global['Message'].Show('Error', f'Location Not Selected In {self.Project_2}')
                    return
            self.Global['Message'].Show('Error', f'Item Not Selected In {self.Project_1}')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
    
    def Copy_From_2_To_1(self):
        try:
            self.Global['Message'].Hide()
            From = self.Element_2.Tree.Current()
            if From:
                Element_ID = self.Element_2.Tree.Get(From)
                if Element_ID[0]!=self.Element_2.Parent:
                    self.From = [Element_ID, 2, From]
                    To = self.Element_1.Tree.Current()
                    if To:
                        Element_ID = self.Element_1.Tree.Get(To)
                        if Element_ID[0]!=self.Element_1.Parent:
                            self.To = [Element_ID, 1, To]
                            self.Copy()
                            return
                    self.Global['Message'].Show('Error', f'Location Not Selected In {self.Project_1}')
                    return
            self.Global['Message'].Show('Error', f'Item Not Selected In {self.Project_2}')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Copy(self):
        try:
            Paste_Type = self.To[0][1]
            if Paste_Type=='Frame' or Paste_Type=='Canvas' or Paste_Type=='Scroll':
                self.Paste(self.From[2], self.To[2])
            else:
                self.Global['Message'].Show('Error', 'Copy To Selected Widget Not Posiible')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Paste(self, ID=False, Root=False):
        try:
            if ID and Root:
                From = getattr(self, f"Element_{self.From[1]}")
                To = getattr(self, f"Element_{self.To[1]}")
                Root_ID = To.Tree.Get(Root)[0]
                Element_ID = From.Tree.Get(ID)[0]
                if len(From.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Frame'
                elif len(From.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Item'
                else:
                    Type = 'Widget'
                Run = True
                if Type == 'Item':
                    if Root_ID!='Root':
                        Run = False
                        Widget = To.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Root_ID}'", Keys=True)
                        while Widget[0]['Type']=='Group' and Widget[0]['Root']!='Root':
                            Temp_Root_ID = Widget[0]['Root']
                            Widget = To.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root_ID}'", Keys=True)
                        if Widget[0]['Type']=='Canvas' or Widget[0]['Type']=='Scroll':
                            Run = True
                    else:
                        Run = False
                if Run:
                    From.Database.Post(f"DELETE FROM `{Type}_Copy`")
                    From.Database.Post(f"INSERT INTO `{Type}_Copy` SELECT * FROM `{Type}` WHERE `ID`='{Element_ID}'")
                    Frame_Data = From.Database.Get(f"SELECT * FROM `{Type}_Copy` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    Number = 0
                    Exist = From.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Frame_Data['Name']}' AND `Root`='{Root_ID}')")
                    while len(Exist)>0:
                        Number += 1
                        Exist = From.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Frame_Data['Name']}{Number}' AND `Root`='{Root_ID}')")
                    if Number>0:
                        Name = f"{Frame_Data['Name']}{Number}"
                    else:
                        Name = Frame_Data['Name']
                    Random_Letter = ''.join(random.choices(string.ascii_letters, k=10))
                    New_ID = Random_Letter+self.Global['Custom'].MD5(Root_ID+Name+str(time.time()*1000000))
                    From.Database.Post(f"UPDATE `{Type}_Copy` SET `ID`='{New_ID}',`Name`='{Name}',`Root`='{Root_ID}' WHERE `ID`='{Element_ID}'")
                    Data = From.Database.Get(f"SELECT * FROM `{Type}_Copy` WHERE `ID`='{New_ID}'", Keys=True)[0]
                    Columns = ','.join(f'`{K}`' for K in Data.keys())
                    Values = ','.join(self.Format(V) for V in Data.values())
                    To.Database.Post(f"INSERT INTO `{Type}` ({Columns}) VALUES ({Values})")
                    From.Database.Post(f"DELETE FROM `{Type}_Copy` WHERE `ID`='{New_ID}'")
                    if os.path.exists(f"{From.Project_Path}/Data/File/{Element_ID}"):
                        shutil.copy(f"{From.Project_Path}/Data/File/{Element_ID}", f"{To.Project_Path}/Data/File/{New_ID}")
                    if Type=='Frame':
                        Root_Level = To.Tree.Get(Root)[2]
                        To.Database.Post(f"UPDATE `{Type}` SET `Level`='{Root_Level+1}' WHERE `ID`='{New_ID}'")
                    Widget = To.Database.Get(f"SELECT * FROM `{Type}` WHERE `ID`='{New_ID}'", Keys=True)[0]
                    Level = 0
                    if Type=='Frame':
                        Level = Widget['Level']
                    Image = self.Global['Image'](Widget['Type'])
                    Temp_ID = To.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], Level], Path=Image)
                    for Each in From.Tree.Child(ID):
                        self.Paste(Each, Temp_ID)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Update(self, Loading=True, Partial=False):
        try:
            if Loading:
                self.Global['Loading'].Show()
                self.Global['GUI'].After(500, lambda:self.Update(Loading=False, Partial=Partial))
            else:
                if not Partial:
                    if self.Database_1:
                        self.Database_1.Close()
                    if self.Database_2:
                        self.Database_2.Close()
                    self.Database_1 = self.Global['Gluonix'].SQL(f'{self.Project_1_Path}/Data/NGD.dll')
                    self.Element_1.Database = self.Database_1
                    self.Element_1.Project_Path = self.Project_1_Path
                    Display_Data = self.Database_1.Get(f"SELECT * FROM `Display`", Keys=True)
                    self.Display_1_Select.Clear()
                    for Display in Display_Data:
                        self.Display_1_Select.Add(Display['ID'])
                        if Display['ID']=='Root':
                            self.Label_1.Set(Display['Title'])
                            self.Project_1 = Display['Title']
                            self.Display_1_Select.Set(Display['ID'])
                    self.Database_2 = self.Global['Gluonix'].SQL(f'{self.Project_2_Path}/Data/NGD.dll')
                    self.Element_2.Database = self.Database_2
                    self.Element_2.Project_Path = self.Project_2_Path
                    Display_Data = self.Database_2.Get(f"SELECT * FROM `Display`", Keys=True)
                    self.Display_2_Select.Clear()
                    for Display in Display_Data:
                        self.Display_2_Select.Add(Display['ID'])
                        if Display['ID']=='Root':
                            self.Label_2.Set(Display['Title'])
                            self.Project_2 = Display['Title']
                            self.Display_2_Select.Set(Display['ID'])
                self.Element_1.Default(self.Display_1_Select.Get())
                self.Element_2.Default(self.Display_2_Select.Get())
                self.Frame.Show()
                self.Global['Loading'].Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Close_Compare(self):
        try:
            if self.Database_1:
                self.Database_1.Close()
            self.Database_1 = False
            if self.Database_2:
                self.Database_2.Close()
            self.Database_2 = False
            self.Element_2.Database = False
            self.Project_1_Path = False
            self.Project_2_Path = False
            self.Element_1.Project_Path = False
            self.Element_2.Project_Path = False
            self.Frame.Hide()
            self.Main.Home.Frame.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Format(self, Value):
        try:
            if isinstance(Value, str):
                return "'" + Value.replace("'", "''") + "'"
            elif Value is None:
                return 'NULL'
            else:
                return str(Value)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Element
################################################################################################################################
import inspect
import os
import shutil
import random
import string
import time
import traceback

class Element:
    def __init__(self, Global, Compare):
        try:
            self.Global = Global
            self.Compare = Compare
            self.Widget = []
            self.Parent = False
            self.Database = False
            self.Project_Path = False
            
            #Frame
            Fixture = self.Compare.Frame.Locate(38, 88, 2, 12)
            self.Frame = self.Global['Gluonix'].Frame(self.Compare.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            self.Compare.Widget.append(self.Frame)
            
            #Delete
            Fixture = self.Frame.Locate(10, 5, 90, 0)
            self.Delete_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Image.Config(Path=self.Global['Image']('Delete'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Delete_Image.Bind(On_Click=lambda E: self.Show_Delete())
            self.Delete_Image.Create()
            
            #Delete Comfirm
            Fixture = self.Frame.Locate(10, 5, 79, 0)
            self.Delete_Confirm_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Confirm_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Confirm_Image.Config(Path=self.Global['Image']('Success'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Delete_Confirm_Image.Bind(On_Click=lambda E: self.Delete())
            self.Delete_Confirm_Image.Create()
            
            #Delete Cancel
            Fixture = self.Frame.Locate(10, 5, 90, 0)
            self.Delete_Cancel_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Cancel_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Cancel_Image.Config(Path=self.Global['Image']('Close'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Delete_Cancel_Image.Bind(On_Click=lambda E: self.Hide_Delete())
            self.Delete_Cancel_Image.Create()
            
            #Tree
            Fixture = self.Frame.Locate(100, 95, 0, 5)
            self.Tree = self.Global['Gluonix'].Tree(self.Frame)
            self.Tree.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Tree.Config(Background='#FFFFFF', Border_Size=0, Scroll_Width=Fixture[0]*3, Display=True)
            self.Tree.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal')
            self.Tree.Config(Resize=True, Move=True)
            self.Tree.Create()
                   
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self):
        try:
            if self.Parent:
                self.Load_Child(self.Parent)
                self.Tree.Expand(getattr(self, self.Parent))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load_Child(self, Parent='Root'):
        try:
            Widgets = self.Database.Get(f"SELECT * FROM `Widget` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Database.Get(f"SELECT * FROM `Item` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Database.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=Widget['Name'], Parent=Root, Value=[Widget['ID'], Widget['Type'], Widget['Level']], Path=Image))
                self.Load_Child(Widget['ID'])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Default(self, Parent='Root'):
        try:
            self.Parent = Parent
            self.Tree.Remove_All()
            setattr(self, self.Parent, self.Tree.Add(self.Parent, Value=[self.Parent, 'Frame', 0]))
            self.Load()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Show_Delete(self):
        try:
            self.Delete_Image.Hide()
            self.Delete_Confirm_Image.Show()
            self.Delete_Cancel_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Hide_Delete(self):
        try:
            self.Delete_Confirm_Image.Hide()
            self.Delete_Cancel_Image.Hide()
            self.Delete_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete(self):
        try:
            ID = self.Tree.Selected()
            for Each in ID:
                self.Delete_All(Each)
            self.Hide_Delete()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_All(self, ID=False):
        try:
            if ID:
                for Each in self.Tree.Child(ID):
                    self.Delete_All(Each)
                Element_ID = self.Tree.Get(ID)[0]
                if Element_ID!=self.Parent:
                    if len(self.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    elif len(self.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    else:
                        Frame_Data = self.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    Temp_Root = Frame_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!=self.Parent:
                        Frame_Data = self.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Frame_Data)==1:
                            Frame_Data = Frame_Data[0]
                            Temp_Root = Frame_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    if os.path.exists(f"{self.Project_Path}/Data/File/{Element_ID}"):
                        os.remove(f"{self.Project_Path}/Data/File/{Element_ID}")
                    self.Tree.Remove(ID)
                    self.Database.Post(f"DELETE FROM `Frame` WHERE `ID`='{Element_ID}'")
                    self.Database.Post(f"DELETE FROM `Item` WHERE `ID`='{Element_ID}'")
                    self.Database.Post(f"DELETE FROM `Widget` WHERE `ID`='{Element_ID}'")
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


# IMPORT LIBRARIES
import tkinter as TK
import threading, math, time
from .N_GUI import GUI
from .N_Custom import Event_Bind
from .N_Custom import Event_Bind_Canvas
from .N_Canvas_Line import Canvas_Line
from .N_Canvas_Polyline import Canvas_Polyline
from .N_Canvas_Pie import Canvas_Pie
from .N_Canvas_Arc import Canvas_Arc
from .N_Canvas_Circle import Canvas_Circle
from .N_Canvas_Oval import Canvas_Oval
from .N_Canvas_Rectangle import Canvas_Rectangle
from .N_Canvas_RectangleR import Canvas_RectangleR
from .N_Canvas_Polygon import Canvas_Polygon
from .N_Canvas_Image import Canvas_Image
from .N_Canvas_Text import Canvas_Text
from .N_Canvas_Text_Old import Canvas_Text_Old
from .N_Player import Player
from .N_Video import Video
from .N_Viewport import Viewport
from .N_Editor import Editor

class Canvas:

    def __init__(self, Main, *args, **kwargs):
        self._GUI = GUI._Instance
        if self._GUI is not None:
            self._Type = "Canvas"
            try:
                try:
                    TK.Canvas.tagraise = TK.Canvas.tkraise
                    del TK.Canvas.tkraise, TK.Canvas.lift
                except Exception:
                    self.Nothing = False
                self._Config = ['Name', 'Auto_Dark', 'Background', 'Light_Background', 'Dark_Background', 'Border_Color', 'Light_Border_Color', 'Dark_Border_Color', 'Border_Size', 'Resize', 'Popup', 'Display', 'Left', 'Top', 'Width', 'Height', 'Animate_Left', 'Animate_Top', 'Animate_Width', 'Animate_Height', 'Animate_Time', 'Radius', 'Shadow_Size', 'Shadow_Color', 'Light_Shadow_Color', 'Dark_Shadow_Color', 'Shadow_Full', 'Hover_Background', 'Light_Hover_Background', 'Dark_Hover_Background', 'Hover_Border_Color', 'Light_Hover_Border_Color', 'Dark_Hover_Border_Color', 'Hover_Shadow_Color', 'Light_Hover_Shadow_Color', 'Dark_Hover_Shadow_Color']
                self._Initialized = False
                self._Name = False
                self._Last_Name = False
                self._Widget = []
                self._Item = []
                self._Resize = True
                self._Popup = False
                self._Display = True
                self._Main = Main
                self._Frame = TK.Canvas(self._Main._Frame)
                self._Border_Color = '#000000'
                self._Border_Size = 0
                self._Border = []
                self._Shadow_Size = 0
                self._Shadow_Color = '#d5d8dc'
                self._Shadow_Full = True
                self._Radius = 0
                self._Hover_Background = False
                self._Hover_Border_Color = False
                self._Hover_Shadow_Color = False
                self._Last_Background = False
                self._Last_Border_Color = False
                self._Last_Shadow_Color = False
                self._Animating = False
                self._Anim_Stop = None
                self._Anim_Thread = None
                self._Animate_Ease = lambda t: (1 - (1 - t)**3)
                self._Animate_Speed = None
                self._Animate_Left = 0
                self._Animate_Top = 0
                self._Animate_Width = 0
                self._Animate_Height = 0
                self._Animate_Time = 1.0
                self._Rounded = []
                self._Background = self._Main._Background
                self._Background_Main = True
                self._On_Resize = False
                self._Auto_Dark = True
                self._On_Show = False
                self._On_Hide = False
                self._On_Animate = False
                self._Configure_After_Id = None
                self._Last_Configure_Event = None
                self._Last_Width = 0
                self._Last_Height = 0
            except Exception as E:
                self._GUI.Error(f"{self._Type} -> Init -> {E}")
        else:
            print("Error: Gluonix -> GUI Instance Has Not Been Created")

    def __str__(self):
        return "Nucleon_Glunoix_Canvas[]"

    def __repr__(self):
        return "Nucleon_Glunoix_Canvas[]"
    
    def Copy(self, Name=False, Main=False):
        try:
            if not Main:
                Main = self._Main
            Instance = type(self)(Main)
            for Key in self._Config:
                if hasattr(self, "_"+Key):
                    setattr(Instance, "_"+Key, getattr(self, "_"+Key))
            if Name:
                setattr(Instance, "_Name", Name)
            Instance.Create()
            for Each in self._Widget:
                Each.Copy(Main=Instance)
            return Instance
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Copy -> {E}")
        
    def Delete(self):
        try:
            self.Animate_Cancel()
            self.Clear()
            self._Main._Widget.remove(self)
            self._Frame.delete(self._Border)
            self._Frame.destroy()
            if self:
                del self
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Delete -> {E}")

    def Clear(self):
        try:
            for Each in list(self._Widget):
                Each.Delete()
            for Each in list(self._Item):
                Each.Delete()
            for Each in self._Frame.find_all():
                self._Frame.delete(Each)
            self.Rounded()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Clear -> {E}")

    def Refresh(self):
        try:
            self._Frame.update_idletasks()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Refresh -> {E}")
            
    def Hide(self):
        try:
            self.Animate_Cancel()
            self._Frame.place_forget()
            self._Display = False
            if self._On_Hide:
                self._On_Hide()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Hide -> {E}")

    def _Build_Place_Args(self, Left, Top, Width, Height):
        try:
            Place_Args = {}
            Parent_Width = float(getattr(self._Main, "_Width", 0) or 0)
            Parent_Height = float(getattr(self._Main, "_Height", 0) or 0)
            if self._Resize and self._Main._Type!='Scroll' and Parent_Width > 0 and Parent_Height > 0:
                Place_Args["relx"] = float(Left) / Parent_Width
                Place_Args["rely"] = float(Top) / Parent_Height
                if Width>0:
                    Place_Args["relwidth"] = float(Width) / Parent_Width
                if Height>0:
                    Place_Args["relheight"] = float(Height) / Parent_Height
            else:
                Place_Args["x"] = int(round(Left))
                Place_Args["y"] = int(round(Top))
                if Width>0:
                    Place_Args["width"] = int(round(Width))
                if Height>0:
                    Place_Args["height"] = int(round(Height))
            return Place_Args
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> _Build_Place_Args -> {E}")
            return {}

    def _Place_Geometry(self, Left, Top, Width, Height):
        try:
            Place_Args = self._Build_Place_Args(Left, Top, Width, Height)
            self._Frame.place(**Place_Args)
            self._Frame.lift()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> _Place_Geometry -> {E}")
            
    def On_Configure(self, Event):
        try:
            if Event.width!=self._Last_Width or Event.height!=self._Last_Height:
                self._Last_Width = Event.width
                self._Last_Height = Event.height
                self._Last_Configure_Event = Event
                if self._Configure_After_Id is not None:
                    self._Frame.after_cancel(self._Configure_After_Id)
                self._Configure_After_Id = self._Frame.after(10, self.On_Configure_Debounced)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Configure -> {E}")
            
    def On_Configure_Debounced(self):
        try:
            self._Configure_After_Id = None
            Event = self._Last_Configure_Event
            if not Event:
                return
            self.Rounded(Event.width, Event.height)
            T = threading.Thread(target=self.On_Configure_Thread, args=(Event,), daemon=True)
            T.start()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Configure_Debounced -> {E}")
    
    def On_Configure_Thread(self, Event):
        try:
            for Each in self._Item:
                try:
                    if Each._Display:
                        Each.Resize(Event)
                except Exception as E:
                    self._GUI.Error(f"{self._Type} -> On_Configure -> {E}")
            if self._On_Resize:
                self._On_Resize(Event)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Configure_Thread -> {E}")
            
    def Show(self):
        try:
            if self._Animating:
                return
            self._Display = True
            self._Place_Geometry(self._Left, self._Top, self._Width, self._Height)
            for Each in self._Widget:
                try:
                    if Each._Display:
                        Each.Show()
                except Exception:
                    self.Nothing = False
            Left, Top, Width, Height = self.Box()
            self._Last_Width = Width
            self._Last_Height = Height
            if self._On_Show:
                self._On_Show()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Show -> {E}")
            
    def Animate(self, Widget=None, Hide=False, Thread=True):
        try:
            self.Animate_Cancel()
            Final_Left = float(self._Left)
            Final_Top = float(self._Top)
            Final_Width = float(self._Width)
            Final_Height = float(self._Height)
            Start_Left = float(self._Animate_Left)
            Start_Top = float(self._Animate_Top)
            Animate_Width = float(getattr(self, "_Animate_Width", 0) or 0)
            Animate_Height = float(getattr(self, "_Animate_Height", 0) or 0)
            Size_Anim = not (Animate_Width == 0 and Animate_Height == 0)
            Start_Width = Animate_Width if Size_Anim else Final_Width
            Start_Height = Animate_Height if Size_Anim else Final_Height
            Same_Pos = int(round(Start_Left)) == int(round(Final_Left)) and int(round(Start_Top)) == int(round(Final_Top))
            Same_Size = int(round(Start_Width)) == int(round(Final_Width)) and int(round(Start_Height)) == int(round(Final_Height))
            if Same_Pos and (not Size_Anim or Same_Size):
                Width_I = int(round(Final_Width))
                Height_I = int(round(Final_Height))
                self.Rounded(Width_I, Height_I)
                self._Place_Geometry(Final_Left, Final_Top, Final_Width, Final_Height)
                self._Display = True
                for Each in self._Widget:
                    try:
                        if Each._Display:
                            Each.Show()
                    except Exception:
                        self.Nothing = False
                if Widget is not None:
                    self._Frame.itemconfigure(Widget, state='normal')
                    self._Frame.tag_raise(Widget)
                    self._Frame.coords(Widget, Width_I/2, Height_I/2)
                if self._On_Animate:
                    self._On_Animate()
                if Hide:
                    self.Hide()
                return
            def Show_Start():
                if not self._Frame.winfo_exists():
                    return
                Width_I = int(round(Start_Width))
                Height_I = int(round(Start_Height))
                self.Rounded(Width_I, Height_I)
                self._Place_Geometry(Start_Left, Start_Top, Start_Width, Start_Height)
                self._Display = True
                for Each in self._Widget:
                    try:
                        if Each._Display:
                            Each.Show()
                    except Exception:
                        self.Nothing = False
                if Widget is not None:
                    self._Frame.itemconfigure(Widget, state='normal')
                    self._Frame.tag_raise(Widget)
                    self._Frame.coords(Widget, Width_I/2, Height_I/2)
            self._Frame.after(0, Show_Start)
            Dx = Final_Left - Start_Left
            Dy = Final_Top - Start_Top
            Dw = Final_Width - Start_Width if Size_Anim else 0.0
            Dh = Final_Height - Start_Height if Size_Anim else 0.0
            Dist = math.hypot(math.hypot(Dx, Dy), math.hypot(Dw, Dh))
            if Dist == 0.0:
                def Snap_Same():
                    if not self._Frame.winfo_exists():
                        return
                    Width_I = int(round(Final_Width))
                    Height_I = int(round(Final_Height))
                    self.Rounded(Width_I, Height_I)
                    self._Place_Geometry(Final_Left, Final_Top, Final_Width, Final_Height)
                    self._Display = True
                    for Each in self._Widget:
                        try:
                            if Each._Display:
                                Each.Show()
                        except Exception:
                            self.Nothing = False
                    if Widget is not None:
                        self._Frame.itemconfigure(Widget, state='normal')
                        self._Frame.tag_raise(Widget)
                        self._Frame.coords(Widget, Width_I/2, Height_I/2)
                    if self._On_Animate:
                        self._On_Animate()
                    if Hide:
                        self.Hide()
                self._Frame.after(0, Snap_Same)
                return
            if self._Animate_Speed and self._Animate_Speed > 0:
                Duration = max(0.001, Dist / float(self._Animate_Speed))
            else:
                Duration = max(0.001, float(self._Animate_Time))
            Ease = self._Animate_Ease or (lambda t: t)
            Target_FPS = 90.0
            Frame_Interval = 1.0 / Target_FPS
            self._Animating = True
            Stop = threading.Event()
            self._Anim_Stop = Stop
            def Worker():
                T0 = time.perf_counter()
                Next_Tick = T0
                Last = None
                while not Stop.is_set():
                    Now = time.perf_counter()
                    T = (Now - T0) / Duration
                    if T >= 1.0:
                        def Snap_Final():
                            if Stop.is_set() or not self._Frame.winfo_exists():
                                return
                            Width_I = int(round(Final_Width))
                            Height_I = int(round(Final_Height))
                            self.Rounded(Width_I, Height_I)
                            self._Place_Geometry(Final_Left, Final_Top, Final_Width, Final_Height)
                            for Each in self._Widget:
                                try:
                                    if Each._Display:
                                        Each.Show()
                                except Exception:
                                    self.Nothing = False
                            self._Animating = False
                            if Widget is not None:
                                self._Frame.itemconfigure(Widget, state='normal')
                                self._Frame.tag_raise(Widget)
                                self._Frame.coords(Widget, Width_I/2, Height_I/2)
                            if Hide:
                                self.Hide()
                            if self._On_Animate:
                                self._On_Animate()
                        self._Frame.after(0, Snap_Final)
                        return
                    K = Ease(max(0.0, min(1.0, T)))
                    X = Start_Left + Dx * K
                    Y = Start_Top + Dy * K
                    W = Start_Width + Dw * K
                    H = Start_Height + Dh * K
                    Cur = (int(round(X)), int(round(Y)), int(round(W)), int(round(H)))
                    if Cur != Last:
                        Last = Cur
                        def Post(C=Cur):
                            if Stop.is_set() or not self._Frame.winfo_exists():
                                return
                            if self._Animating:
                                Width_I = C[2]
                                Height_I = C[3]
                                self.Rounded(Width_I, Height_I)
                                self._Place_Geometry(C[0], C[1], C[2], C[3])
                                for Each in self._Widget:
                                    try:
                                        if Each._Display:
                                            Each.Show()
                                    except Exception:
                                        self.Nothing = False
                                if Widget is not None:
                                    self._Frame.itemconfigure(Widget, state='normal')
                                    self._Frame.tag_raise(Widget)
                                    self._Frame.coords(Widget, Width_I/2, Height_I/2)
                        self._Frame.after(0, Post)
                    Next_Tick += Frame_Interval
                    Sleep_For = Next_Tick - time.perf_counter()
                    if Sleep_For < -2 * Frame_Interval:
                        Next_Tick = time.perf_counter()
                        Sleep_For = Frame_Interval
                    if Sleep_For > 0:
                        if Stop.wait(Sleep_For):
                            return
            self._Display = True
            if Thread:
                T = threading.Thread(target=Worker, daemon=True)
                self._Anim_Thread = T
                T.start()
            else:
                Worker()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Animate -> {E}")
            self.Animate_Cancel()
        
    def Animate_Cancel(self):
        try:
            self._Animating = False
            if self._Anim_Stop is not None:
                self._Anim_Stop.set()
            self._Anim_Stop = None
            self._Anim_Thread = None
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Animate_Cancel -> {E}")

    def Focus(self):
        try:
            self._Frame.focus_set()
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Focus -> {E}")
    
    def Grab(self, Path=False):
        try:
            return self._GUI.Grab_Widget(Path=Path, Widget=self)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Grab -> {E}")
            
    def Widget(self):
        try:
            return self._Frame
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Widget -> {E}")
            
    def Bind(self, **Input):
        try:
            if 'On_Show' in Input:
                self._On_Show = Input['On_Show']
            if 'On_Hide' in Input:
                self._On_Hide = Input['On_Hide']
            if 'On_Animate' in Input:
                self._On_Animate = Input['On_Animate']
            if "On_Resize" in Input:
                self._On_Resize = Input["On_Resize"]
            Event_Bind(self._Frame, **Input)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Bind -> {E}")
            
    def Bind_Item(self, Item, **Input):
        try:
            Event_Bind_Canvas(self._Frame, Item, **Input)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Bind_Item -> {E}")
            
    def On_Hover_In(self, E):
        try:
            Config = {}
            if self._Hover_Background:
                self._Last_Background = self._Background
                Config['Background'] = self._Hover_Background
            if self._Hover_Border_Color:
                self._Last_Border_Color = self._Border_Color
                Config['Border_Color'] = self._Hover_Border_Color
            if self._Hover_Shadow_Color:
                self._Last_Shadow_Color = self._Shadow_Color
                Config['Shadow_Color'] = self._Hover_Shadow_Color
            if len(Config)>0:
                self.Config(**Config)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Hover_In -> {E}")
            
    def On_Hover_Out(self, E):
        try:
            Config = {}
            if self._Hover_Background and self._Last_Background:
                Config['Background'] = self._Last_Background if self._Background==self._Hover_Background else self._Background
            if self._Hover_Border_Color and self._Last_Border_Color:
                Config['Border_Color'] = self._Last_Border_Color if self._Border_Color==self._Hover_Border_Color else self._Border_Color
            if self._Hover_Shadow_Color and self._Last_Shadow_Color:
                Config['Shadow_Color'] = self._Last_Shadow_Color if self._Shadow_Color==self._Hover_Shadow_Color else self._Shadow_Color
            if len(Config)>0:
                self.Config(**Config)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> On_Hover_Out -> {E}")
            
    def Config_Get(self, *Input):
        try:
            Return = {}
            for Each in self._Config:
                if Each in Input:
                    Return[Each] = getattr(self, "_"+Each)
            return Return
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Config_Get -> {E}")
                
    def Config(self, **Input):
        try:
            Run = False
            for Each in self._Config:
                if Each in Input:
                    Value = Input[Each]
                    setattr(self, "_"+Each, Value)
                    Run = True
            if self._Initialized and Run:
                self.Create()
            if "Background" in Input:
                self._Background_Main = not bool(Input["Background"])
                for Each in self._Widget:
                    try:
                        if Each._Background_Main:
                            Each.Config(Background=False)
                        if hasattr(Each, '_Radius'):
                            if Each._Radius>0:
                                Each.Config(Radius=Each._Radius)
                    except Exception:
                        self.Nothing = False
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Config -> {E}")
            
    def Move(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left += Left
            if Top is not None:
                self._Top += Top
            if Left is not None or Top is not None:
                self.Position(Left=self._Left, Top=self._Top)
            return True
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Move -> {E}")
            
    def Center(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left = Left-self._Width/2
            if Top is not None:
                self._Top = Top-self._Height/2
            if Left is not None or Top is not None:
                self.Position(Left=self._Left, Top=self._Top)
            return [self._Left+self._Width/2, self._Top+self._Height/2]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Center -> {E}")
            
    def Position(self, Left=None, Top=None):
        try:
            if Left is not None:
                self._Left = Left
            if Top is not None:
                self._Top = Top
            if Left is not None or Top is not None:
                if self._Display and not self._Animating:
                    self._Place_Geometry(self._Left, self._Top, 0, 0)
            return [int(self._Left), int(self._Top)]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Position -> {E}")
            
    def Size(self, Width=False, Height=False):
        try:
            if Width:
                self._Width = Width
            if Height:
                self._Height = Height
            if Width or Height:
                if self._Display and not self._Animating:
                    self._Place_Geometry(self._Left, self._Top, self._Width, self._Height)
            return [int(self._Width), int(self._Height)]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Size -> {E}")
    
    def Enlarge(self, Value=None):
        try:
            if Value is not None:
                self._Left -= Value
                self._Top -= Value
                self._Width += Value
                self._Height += Value
                if self._Initialized and self._Display and not self._Animating:
                    self._Place_Geometry(self._Left, self._Top, self._Width, self._Height)
            return True
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Enlarge -> {E}")
    
    def Shrink(self, Value=None):
        try:
            if Value is not None:
                self._Left += Value
                self._Top += Value
                self._Width -= Value
                self._Height -= Value
                if self._Initialized and self._Display and not self._Animating:
                    self._Place_Geometry(self._Left, self._Top, self._Width, self._Height)
            return True
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Shrink -> {E}")
            
    def Box(self):
        try:
            return [self._Frame.winfo_x(), self._Frame.winfo_y(), self._Frame.winfo_width(), self._Frame.winfo_height()]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Box -> {E}")
        
    def Ratio(self):
        try:
            Width_Ratio = self._Frame.winfo_width()/self._Width
            Height_Ratio = self._Frame.winfo_height()/self._Height
            return [Width_Ratio, Height_Ratio]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Ratio -> {E}")
        
    def Locate(self, Width, Height, Left, Top):
        try:
            Width = self._Width*(Width/100)
            Height = self._Height*(Height/100)
            Left = self._Width*(Left/100)
            Top = self._Height*(Top/100)
            return [Width, Height, Left, Top]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Locate -> {E}")
        
    def Locate_Reverse(self, Width, Height, Left, Top):
        try:
            Width = round((Width/self._Width)*100, 3)
            Height = round((Height/self._Height)*100, 3)
            Left =  round((Left/self._Width)*100, 3)
            Top =  round((Top/self._Height)*100, 3)
            return [Width, Height, Left, Top]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Locate_Reverse -> {E}")
            
    def Create(self):
        try:
            First_Init = not self._Initialized
            if not self._Background:
                self._Background = self._Main._Background
                if not hasattr(self, "_Light_Background"):
                    setattr(self, "_Light_Background", self._Background)
                if not hasattr(self, "_Dark_Background"):
                    setattr(self, "_Dark_Background", self._GUI.Invert(self._Background))
            Temp_Background = self._Background
            if self._Auto_Dark and not self._GUI._Dark_Mode:
                self.Update_Color()
            if First_Init:
                self.Update_Color()
                if not self._Display:
                    self.Hide()
                self._Main._Widget.append(self)
                Event_Bind(self._Frame, On_Hover_In=lambda E: self.On_Hover_In(E), On_Hover_Out=lambda E: self.On_Hover_Out(E))
                self._Frame.bind("<Configure>", self.On_Configure, add="+")
                self._Initialized = True
            if self._Radius:
                Temp_Background = self._Main._Background
            self._Frame.config(background=Temp_Background, width=self._Width, height=self._Height, highlightthickness=0)
            self.Rounded()
            if self._Display:
                self.Show()
            else:
                self.Hide()
            if self._Name!=self._Last_Name:
                if self._Last_Name:
                    if self._Last_Name in self._Main.__dict__:
                        del self._Main.__dict__[self._Last_Name]
                if self._Name:
                    self._Main.__dict__[self._Name] = self
                self._Last_Name = self._Name
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Create -> {E}")
            
    def Update_Color(self):
        try:
            self._GUI.Initiate_Colors(self)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Update_Color -> {E}")
            
    def Rounded(self, Width=False, Height=False):
        try:
            for Each in self._Rounded:
                self._Frame.delete(Each)
            self._Rounded = []
            if not Width:
                Width = getattr(self, "_Width", 0)
            if not Height:
                Height = getattr(self, "_Height", 0)
            Radius = min(self._Radius, Width / 2, Height / 2) if Width>0 and Height>0 else 0
            X1 = self._Shadow_Size
            Y1 = self._Shadow_Size
            X2 = Width-self._Shadow_Size-1
            Y2 = Height-self._Shadow_Size-1
            Reducer = 0
            if not self._Shadow_Full:
                Reducer = self._Shadow_Size
            for i in range(self._Shadow_Size):
                Offset = self._Shadow_Size-i
                Intensity = (self._Shadow_Size-i) / self._Shadow_Size if self._Shadow_Size>0 else 0
                Shadow_Fill = self.Fade(self._Shadow_Color, Intensity) if self._Shadow_Size>0 else self._Shadow_Color
                self._Rounded.append(self.Round_Rectangle(X1-Offset+(Reducer/2), Y1-Offset+Reducer, X2+Offset-(Reducer/2), Y2+Offset, Radius, outline=Shadow_Fill, fill=Shadow_Fill, smooth=True))
            if self._Border_Size>0 and Width>0 and Height>0:
                self._Rounded.append(self.Round_Rectangle(X1, Y1, X2, Y2, Radius, outline=self._Border_Color, fill=self._Border_Color, smooth=True))
            X1 = self._Border_Size+self._Shadow_Size
            Y1 = self._Border_Size+self._Shadow_Size
            X2 = Width-self._Border_Size-self._Shadow_Size-1
            Y2 = Height-self._Border_Size-self._Shadow_Size-1
            if Width>0 and Height>0:
                self._Rounded.append(self.Round_Rectangle(X1, Y1, X2, Y2, Radius, outline=self._Background, fill=self._Background, smooth=True))
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Rounded -> {E}")
            
    def RGB(self, Color):
        try:
            return [int(Color[i:i+2], 16) for i in (1, 3, 5)]
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> RGB -> {E}")
    
    def HEX(self, Color):
        try:
            return f'#{Color[0]:02x}{Color[1]:02x}{Color[2]:02x}'
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> HEX -> {E}")

    def Fade(self, Color, Intensity):
        try:
            R1, G1, B1 = self.RGB(Color)
            R2, G2, B2 = self.RGB(self._Main._Background)
            R = int(R1 + (R2 - R1) * Intensity)
            G = int(G1 + (G2 - G1) * Intensity)
            B = int(B1 + (B2 - B1) * Intensity)
            return self.HEX([R, G, B])
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> FADE -> {E}")
            
    def Points(self, Radius):
        try:
            Radius = math.ceil(Radius)
            if Radius<1:
                Radius = 1
            for i in range(Radius + 1):
                Angle = math.pi * (i / Radius) * 0.5
                X = (math.cos(Angle) - 1) * Radius
                Y = (math.sin(Angle) - 1) * Radius
                yield (X, Y)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Points -> {E}")
            
    def Round_Rectangle(self, X1, Y1, X2, Y2, Radius, **Args):
        try:
            if Radius>1:
                Points = []
                Cos_Sin_R = tuple(self.Points(Radius))
                for Cos_R, Sin_R in Cos_Sin_R:
                    Points.append((X2 + Sin_R, Y1 - Cos_R))
                for Cos_R, Sin_R in Cos_Sin_R:
                    Points.append((X2 + Cos_R, Y2 + Sin_R))
                for Cos_R, Sin_R in Cos_Sin_R:
                    Points.append((X1 - Sin_R, Y2 + Cos_R))
                for Cos_R, Sin_R in Cos_Sin_R:
                    Points.append((X1 - Cos_R, Y1 - Sin_R))
                return self._Frame.create_polygon(Points, **Args)
            else:
                del Args['smooth']
                return self._Frame.create_rectangle(X1, Y1, X2, Y2, **Args)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Round_Rectangle -> {E}")
            
    def Bbox(self, Item=False):
        try:
            if Item:
                return self._Frame.bbox(Item)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Position -> {E}")

    def Hide_Item(self, Item=False):
        try:
            if Item:
                self._Frame.itemconfigure(Item, state='hidden')
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Hide_Item -> {E}")

    def Show_Item(self, Item=False):
        try:
            if Item:
                self._Frame.itemconfigure(Item, state='normal')
                self._Frame.tag_raise(Item)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Show_Item -> {E}")
    
    def Delete_Item(self, Item=False):
        try:
            if Item:
                self._Frame.delete(Item)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Delete_Item -> {E}")
    
    def Delete_All(self):
        try:
            self._Frame.delete('all')
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Delete_All -> {E}")
            
    def Find_Near(self, X, Y):
        try:
            return self._Frame.find_closest(X, Y)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Find_Near -> {E}")
            
    def Find_Overlap(self, X1, Y1, X2, Y2):
        try:
            return self._Frame.find_overlapping(X1, Y1, X2, Y2)
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Find_Overlap -> {E}")
            
    def Line(self, Name=False):
        try:
            Item = Canvas_Line(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Line -> {E}")
            
    def Polyline(self, Name=False):
        try:
            Item = Canvas_Polyline(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Polyline -> {E}")
                
    def Pie(self, Name=False):
        try:
            Item = Canvas_Pie(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Pie -> {E}")
                
    def Arc(self, Name=False):
        try:
            Item = Canvas_Arc(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Arc -> {E}")
                
    def Circle(self, Name=False):
        try:
            Item = Canvas_Circle(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Circle -> {E}")
                
    def Oval(self, Name=False):
        try:
            Item = Canvas_Oval(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Oval -> {E}")
            
    def Polygon(self, Name=False):
        try:
            Item = Canvas_Polygon(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Polygon -> {E}")
                
    def Rectangle(self, Name=False):
        try:
            Item = Canvas_Rectangle(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Rectangle -> {E}")
                
    def RectangleR(self, Name=False):
        try:
            Item = Canvas_RectangleR(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> RectangleR -> {E}")
                
    def Image(self, Name=False):
        try:
            Item = Canvas_Image(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Image -> {E}")
            
    def Text(self, Name=False):
        try:
            Item = Canvas_Text(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Text -> {E}")
            
    def Text_Old(self, Name=False):
        try:
            Item = Canvas_Text_Old(self)
            if Name:
                Item.Config(Name=Name)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Text_Old -> {E}")
            
    def Player(self):
        try:
            Item = Player(self)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Player -> {E}")
            
    def Video(self):
        try:
            Item = Video(self)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Video -> {E}")
            
    def Viewport(self):
        try:
            Item = Viewport(self)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Viewport -> {E}")
            
    def Editor(self):
        try:
            Item = Editor(self)
            return Item
        except Exception as E:
            self._GUI.Error(f"{self._Type} -> Editor -> {E}")


################################################################################################################################
#Menu
################################################################################################################################
import os
import inspect
import shutil
import time

class Compare:
    def __init__(self, Global, Project):
        try:
            self.Global = Global
            self.Project = Project
            self.Widget = []
            self.Error_1 = False
            self.Error_2 = False
            
            Fixture = self.Global['GUI'].Locate(60, 50, 25, 25)
            self.Frame = self.Global['Gluonix'].Frame(self.Global['GUI'])
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='white', Border_Size=1, Display=False)
            self.Frame.Config(Animate_Left=Fixture[2], Animate_Top=-1*(Fixture[1]+Fixture[3]), Animate_Time=0.5)
            self.Frame.Create()
            self.Project.Widget.append(self.Frame)
            
                #Info Label
            Fixture = self.Frame.Locate(70, 6, 3, 8)
            self.Info_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Info_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Info_Label.Config(Foreground='black', Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Info_Label.Config(Value="Select two Gluonix projects to compare")
            self.Info_Label.Create()
            
                #Project 1 Label
            Fixture = self.Frame.Locate(10, 6, 3, 24)
            self.Project_1_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Project_1_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_1_Label.Config(Foreground='black', Value="Project 01:", Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Project_1_Label.Create()
            
                #Project 1 Entry
            Fixture = self.Frame.Locate(70, 6, 13, 24)
            self.Project_1_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Project_1_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_1_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='left', Border_Size=1)
            self.Project_1_Entry.Bind(On_Key_Release=lambda E: self.Project_1_Update_Path())
            self.Project_1_Entry.Config(Hover_Background='#EEEEEE', Hover_Border_Color='#0078d7')
            self.Project_1_Entry.Create()
            
                #Project 1 Browser
            Fixture = self.Frame.Locate(12, 6, 85, 24)
            self.Project_1_Browser = self.Global['Gluonix'].Label(self.Frame)
            self.Project_1_Browser.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_1_Browser.Config(Background='#e1e1e1', Foreground='black', Value='Browser...', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Project_1_Browser.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Project_1_Browser.Bind(On_Click=lambda E: self.Project_1_File_Select())
            self.Project_1_Browser.Create()
            
                #Project 2 Label
            Fixture = self.Frame.Locate(10, 6, 3, 40)
            self.Project_2_Label = self.Global['Gluonix'].Label(self.Frame)
            self.Project_2_Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_2_Label.Config(Foreground='black', Value="Project 02:", Font_Size=9, Font_Weight='normal', Align='w', Border_Size=0)
            self.Project_2_Label.Create()
            
                #Project 2 Entry
            Fixture = self.Frame.Locate(70, 6, 13, 40)
            self.Project_2_Entry = self.Global['Gluonix'].Entry(self.Frame)
            self.Project_2_Entry.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_2_Entry.Config(Background='white', Foreground='black', Font_Size=10, Font_Weight='normal', Align='left', Border_Size=1)
            self.Project_2_Entry.Bind(On_Key_Release=lambda E: self.Project_2_Update_Path())
            self.Project_2_Entry.Config(Hover_Background='#EEEEEE', Hover_Border_Color='#0078d7')
            self.Project_2_Entry.Create()
            
                #Project 2 Browser
            Fixture = self.Frame.Locate(12, 6, 85, 40)
            self.Project_2_Browser = self.Global['Gluonix'].Label(self.Frame)
            self.Project_2_Browser.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Project_2_Browser.Config(Background='#e1e1e1', Foreground='black', Value='Browser...', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Project_2_Browser.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Project_2_Browser.Bind(On_Click=lambda E: self.Project_2_File_Select())
            self.Project_2_Browser.Create()
            
                #Ok Button
            Fixture = self.Frame.Locate(12, 6, 70, 88)
            self.Ok_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Ok_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Ok_Button.Config(Background='#e1e1e1', Foreground='black', Value='OK', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Ok_Button.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Ok_Button.Bind(On_Click=lambda E: self.Create())
            self.Ok_Button.Create()
            
                #Cancel Button
            Fixture = self.Frame.Locate(12, 6, 85, 88)
            self.Cancel_Button = self.Global['Gluonix'].Label(self.Frame)
            self.Cancel_Button.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Cancel_Button.Config(Background='#e1e1e1', Foreground='black', Value='Cancel', Font_Size=9, Font_Weight='normal', Border_Size=1, Border_Color='#adadad')
            self.Cancel_Button.Config(Hover_Background='#d5dcf0', Hover_Border_Color='#0078d7')
            self.Cancel_Button.Bind(On_Click=lambda E: self.Frame.Hide())
            self.Cancel_Button.Create()
                
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Reset(self):
        try:
            self.Project_1_Entry.Set('')
            self.Project_2_Entry.Set('')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Project_1_File_Select(self):
        try:
            Project_File_Path = self.Global['GUI'].File(Initial=os.path.join(os.path.expanduser('~'), 'Documents'), Title='Select Project', Default='.py', Type=[["Python (*.py)", "*.py"], ["Nucleon Gluonix (*.ng)", "*.ng"]])
            if Project_File_Path:
                self.Project_1_Entry.Set(Project_File_Path)
                self.Project_1_Update_Path()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
            
    def Project_1_Update_Path(self):
        try:
            if os.path.exists(self.Project_1_Entry.Get()):
                self.Project_1_Entry.Config(Border_Color='black')
                self.Error_1 = False
            else:
                self.Project_1_Entry.Config(Border_Color='red')
                self.Error_1 = True
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Project_2_File_Select(self):
        try:
            Project_File_Path = self.Global['GUI'].File(Initial=os.path.join(os.path.expanduser('~'), 'Documents'), Title='Select Project', Default='.py', Type=[["Python (*.py)", "*.py"], ["Nucleon Gluonix (*.ng)", "*.ng"]])
            if Project_File_Path:
                self.Project_2_Entry.Set(Project_File_Path)
                self.Project_2_Update_Path()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
            
    def Project_2_Update_Path(self):
        try:
            if os.path.exists(self.Project_2_Entry.Get()):
                self.Project_2_Entry.Config(Border_Color='black')
                self.Error_2 = False
            else:
                self.Project_2_Entry.Config(Border_Color='red')
                self.Error_2 = True
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        
    def Create(self):
        try:
            Project_1_File_Path = self.Project_1_Entry.Get()
            Project_2_File_Path = self.Project_2_Entry.Get()
            if not self.Error_1 and not self.Error_2:
                self.Global['Message'].Hide()
                Project_1_Path = os.path.dirname(Project_1_File_Path)
                Project_2_Path = os.path.dirname(Project_2_File_Path)
                if Project_1_File_Path.endswith('.py'):
                    Project_1_Path = f'{Project_1_Path}/Nucleon'
                if Project_2_File_Path.endswith('.py'):
                    Project_2_Path = f'{Project_2_Path}/Nucleon'
                self.Project.Update_Database(f'{Project_1_Path}/Data/NGD.dll', self.Global['Data']('NGD.dll'))
                self.Project.Update_Database(f'{Project_2_Path}/Data/NGD.dll', self.Global['Data']('NGD.dll'))
                self.Project.Home.Main.Compare.Project_1_Path = Project_1_Path
                self.Project.Home.Main.Compare.Project_2_Path = Project_2_Path
                self.Project.Home.Main.Compare.Update()
                self.Global['Message'].Hide()
                self.Frame.Hide()
            else:
                if self.Error_1:
                    self.Global['Message'].Show('Error', 'Project 01 Does Not Exist')
                if self.Error_2:
                    self.Global['Message'].Show('Error', 'Project 02 Does Not Exist')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))


################################################################################################################################
#Element
################################################################################################################################
import inspect
import os
import re
import shutil
import random
import string
import time
import traceback

class Element:
    def __init__(self, Global, Design):
        try:
            self.Global = Global
            self.Design = Design
            self.Window = False
            self.Widget = []
            self.Grid_Lock = True
            self.Grid_Width = 10
            self.Grid_Height = 10
            self.Copy_ID = False
            self.Current = False
            self.Dragging = False
            self.Current = False
            self.Selected = []
            self._Coord = {'X': 0, 'Y': 0}
            self.Element_Fixture = {}
            
            #Frame
            Fixture = self.Design.Frame.Locate(29.5, 90, 30, 10)
            self.Frame = self.Global['Gluonix'].Frame(self.Design.Frame)
            self.Frame.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Frame.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Frame.Config(Resize=True, Move=True)
            self.Frame.Create()
            self.Design.Widget.append(self.Frame)
            
            #Label
            Fixture = self.Frame.Locate(30, 3, 0, 1)
            self.Label = self.Global['Gluonix'].Label(self.Frame)
            self.Label.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Label.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Label.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal', Align='w')
            self.Label.Config(Resize=True, Move=True)
            self.Label.Set(' PROJECT')
            self.Label.Create()
            
            #Paste
            Fixture = self.Frame.Locate(10, 4, 60, 0.5)
            self.Paste_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Paste_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Paste_Image.Config(Path=self.Global['Image']('Paste'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Paste_Image.Bind(On_Click=lambda E: self.Paste())
            self.Paste_Image.Bind(On_Hover_In=lambda E: self.Paste_Image.Enlarge(2))
            self.Paste_Image.Bind(On_Hover_Out=lambda E: self.Paste_Image.Shrink(2))
            self.Paste_Image.Create()
            
            #Copy
            Fixture = self.Frame.Locate(10, 4, 70, 0.5)
            self.Copy_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Copy_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Copy_Image.Config(Path=self.Global['Image']('Copy'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Copy_Image.Bind(On_Click=lambda E: self.Copy())
            self.Copy_Image.Bind(On_Hover_In=lambda E: self.Copy_Image.Enlarge(2))
            self.Copy_Image.Bind(On_Hover_Out=lambda E: self.Copy_Image.Shrink(2))
            self.Copy_Image.Create()
            
            #Delete Comfirm
            Fixture = self.Frame.Locate(10, 4, 70, 0.5)
            self.Delete_Confirm_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Confirm_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Confirm_Image.Config(Path=self.Global['Image']('Success'), Border_Size=0, Hand_Cursor=True, Display=False)
            self.Delete_Confirm_Image.Bind(On_Click=lambda E: self.Delete())
            self.Delete_Confirm_Image.Bind(On_Hover_In=lambda E: self.Delete_Confirm_Image.Enlarge(2))
            self.Delete_Confirm_Image.Bind(On_Hover_Out=lambda E: self.Delete_Confirm_Image.Shrink(2))
            self.Delete_Confirm_Image.Create()
            
            #Delete
            Fixture = self.Frame.Locate(10, 4, 80, 0.5)
            self.Delete_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Delete_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Delete_Image.Config(Path=self.Global['Image']('Delete'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Delete_Image.Bind(Add=False, On_Click=lambda E: self.Show_Delete())
            self.Delete_Image.Bind(On_Hover_In=lambda E: self.Delete_Image.Enlarge(2))
            self.Delete_Image.Bind(On_Hover_Out=lambda E: self.Delete_Image.Shrink(2))
            self.Delete_Image.Create()
            
            #Grid
            Fixture = self.Frame.Locate(10, 3.5, 90, 0.75)
            self.Grid_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Grid_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Grid_Image.Config(Path=self.Global['Image']('Grid_Lock'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Grid_Image.Bind(On_Click=lambda E: self.Grid())
            self.Grid_Image.Bind(On_Hover_In=lambda E: self.Grid_Image.Enlarge(2))
            self.Grid_Image.Bind(On_Hover_Out=lambda E: self.Grid_Image.Shrink(2))
            self.Grid_Image.Create()
            
            #Align Left
            Fixture = self.Frame.Locate(10, 4.5, 50, 5)
            self.Align_Left_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Align_Left_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Align_Left_Image.Config(Path=self.Global['Image']('Align_Left'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Align_Left_Image.Bind(On_Click=lambda E: self.Align_Left())
            self.Align_Left_Image.Bind(On_Hover_In=lambda E: self.Align_Left_Image.Enlarge(2))
            self.Align_Left_Image.Bind(On_Hover_Out=lambda E: self.Align_Left_Image.Shrink(2))
            self.Align_Left_Image.Create()
            
            #Align Top
            Fixture = self.Frame.Locate(10, 4.5, 60, 5)
            self.Align_Top_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Align_Top_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Align_Top_Image.Config(Path=self.Global['Image']('Align_Top'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Align_Top_Image.Bind(On_Click=lambda E: self.Align_Top())
            self.Align_Top_Image.Bind(On_Hover_In=lambda E: self.Align_Top_Image.Enlarge(2))
            self.Align_Top_Image.Bind(On_Hover_Out=lambda E: self.Align_Top_Image.Shrink(2))
            self.Align_Top_Image.Create()
            
            #Align Right
            Fixture = self.Frame.Locate(10, 4.5, 70, 5)
            self.Align_Right_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Align_Right_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Align_Right_Image.Config(Path=self.Global['Image']('Align_Right'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Align_Right_Image.Bind(On_Click=lambda E: self.Align_Right())
            self.Align_Right_Image.Bind(On_Hover_In=lambda E: self.Align_Right_Image.Enlarge(2))
            self.Align_Right_Image.Bind(On_Hover_Out=lambda E: self.Align_Right_Image.Shrink(2))
            self.Align_Right_Image.Create()
            
            #Align Bottom
            Fixture = self.Frame.Locate(10, 4.5, 80, 5)
            self.Align_Bottom_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Align_Bottom_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Align_Bottom_Image.Config(Path=self.Global['Image']('Align_Bottom'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Align_Bottom_Image.Bind(On_Click=lambda E: self.Align_Bottom())
            self.Align_Bottom_Image.Bind(On_Hover_In=lambda E: self.Align_Bottom_Image.Enlarge(2))
            self.Align_Bottom_Image.Bind(On_Hover_Out=lambda E: self.Align_Bottom_Image.Shrink(2))
            self.Align_Bottom_Image.Create()
            
            #Align Center
            Fixture = self.Frame.Locate(10, 4.5, 90, 5)
            self.Align_Center_Image = self.Global['Gluonix'].Image(self.Frame)
            self.Align_Center_Image.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Align_Center_Image.Config(Path=self.Global['Image']('Align_Center'), Border_Size=0, Hand_Cursor=True, Display=True)
            self.Align_Center_Image.Bind(On_Click=lambda E: self.Align_Center())
            self.Align_Center_Image.Bind(On_Hover_In=lambda E: self.Align_Center_Image.Enlarge(2))
            self.Align_Center_Image.Bind(On_Hover_Out=lambda E: self.Align_Center_Image.Shrink(2))
            self.Align_Center_Image.Create()
            
            #Tree
            Fixture = self.Frame.Locate(100, 90, 0, 10)
            self.Tree = self.Global['Gluonix'].Tree(self.Frame)
            self.Tree.Config(Width=Fixture[0], Height=Fixture[1], Left=Fixture[2], Top=Fixture[3])
            self.Tree.Config(Background='#FFFFFF', Border_Size=0, Display=True)
            self.Tree.Config(Foreground='#000000', Font_Size=16, Font_Weight='normal')
            self.Tree.Config(Resize=True, Move=True)
            self.Tree.Bind(On_Click=lambda E: self.Clear_Selection())
            self.Tree.Bind(On_Release=lambda E: self.Get_Current())
            self.Tree.Bind(On_Control_Click=lambda E: None)
            self.Tree.Bind(On_Control_Release=lambda E: self.Get_Control_Current())
            self.Tree.Create()
                   
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load(self):
        try:
            self.Load_Child(self.Parent)
            self.Tree.Expand(getattr(self, self.Parent))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Load_Child(self, Parent='Root'):
        try:
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=f" {Widget['Name']}", Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=f" {Widget['Name']}", Parent=Root, Value=[Widget['ID'], Widget['Type'], 0], Path=Image))
            Widgets = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `Root`='{Parent}'", Keys=True)
            for Widget in Widgets:
                Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                Stock.Create(Widget['ID'])
                Root = getattr(self, Widget['Root'])
                Image = self.Global['Image'](Widget['Type'])
                setattr(self, Widget['ID'], self.Tree.Add(Name=f" {Widget['Name']}", Parent=Root, Value=[Widget['ID'], Widget['Type'], Widget['Level']], Path=Image))
                self.Load_Child(Widget['ID'])
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Intractive(self, Element):
        try:
            Element.Bind(On_Click=lambda E: self.Drag_Start(E, Element), On_Drag=lambda E: self.Drag(E, Element), On_Release=lambda E: self.Drag_Release(E, Element))
            Element.Bind(On_Right_Click=lambda E: self.Drag_Start_Size(E, Element), On_Right_Drag=lambda E: self.Drag_Size(E, Element), On_Right_Release=lambda E: self.Drag_Release(E, Element))
            Element.Bind(On_Double_Click=lambda E: self.Show_Current(Element))
            Element.Bind(On_Control_Click=lambda E: self.Select_Current(Element))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Drag_Release(self, E, Element):
        try:
            if self.Dragging and self.Current == Element._ID:
                Moved = False
                if hasattr(self, '_Coord'):
                    Dist_X = abs(E.x_root - self._Coord['X'])
                    Dist_Y = abs(E.y_root - self._Coord['Y'])
                    if Dist_X > 2 or Dist_Y > 2:
                        Moved = True
                if not Moved:
                    if len(self.Selected) > 1 and Element in self.Selected:
                         self.Clear_Selection(Element)
            self.Dragging = False
            self.Current = False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Drag_Start(self, E, Element):
        try:
            if Element not in self.Selected:
                self.Clear_Selection(Element)
            if not Element.Lock and not self.Dragging:
                self.Drag_Items = []
                if Element in self.Selected:
                    Reference_Main = self.Selected[0]._Main
                    Same_Parent = True
                    for Item in self.Selected:
                        if Item._Main != Reference_Main:
                            Same_Parent = False
                            break
                    if Same_Parent:
                        self.Drag_Items = self.Selected
                    else:
                        self.Global['Message'].Show('Error', 'Move Same Container Elements Only')
                        return
                else:
                    self.Drag_Items = [Element]
                Any_Locked = False
                for Item in self.Drag_Items:
                    if Item.Lock:
                        Any_Locked = True
                        break
                if Any_Locked:
                    self.Global['Message'].Show('Error', 'Selection Contains Locked Elements')
                    self.Drag_Items = []
                    return
                self._Coord = {'X': E.x_root, 'Y': E.y_root}
                self.Element_Fixtures = {}
                self.Element_Ratios = {}
                for Item in self.Drag_Items:
                    self.Element_Fixtures[Item._ID] = Item.Config_Get('Left', 'Top')
                    Box = Item.Box()
                    Config = Item.Config_Get('Width', 'Height')
                    W_Ratio = 1.0
                    H_Ratio = 1.0
                    if Config['Width'] > 0 and Config['Height'] > 0 and Box[2] > 0 and Box[3] > 0:
                        W_Ratio = Box[2] / Config['Width']
                        H_Ratio = Box[3] / Config['Height']
                    self.Element_Ratios[Item._ID] = {'W': W_Ratio, 'H': H_Ratio}
                self.Dragging = True
                self.Current = Element._ID
                self.Window = getattr(self.Design, self.Parent)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()

    def Drag(self, E, Element):
        try:
            if not Element.Lock and self.Dragging and self.Current==Element._ID:
                Raw_Diff_X = E.x_root - self._Coord['X']
                Raw_Diff_Y = E.y_root - self._Coord['Y']
                for Item in self.Drag_Items:
                    if not Item.Lock and Item._ID in self.Element_Fixtures and Item._ID in self.Element_Ratios:
                        Fixture = self.Element_Fixtures[Item._ID]
                        Ratios = self.Element_Ratios[Item._ID]
                        Difference_Left = Raw_Diff_X / Ratios['W']
                        Difference_Top = Raw_Diff_Y / Ratios['H']
                        New_Left = Fixture['Left'] + Difference_Left
                        New_Top = Fixture['Top'] + Difference_Top
                        if self.Grid_Lock:
                            New_Left = round(New_Left / self.Grid_Width)*self.Grid_Width
                            New_Top = round(New_Top / self.Grid_Height)*self.Grid_Height
                        Item.Position(Left=New_Left, Top=New_Top)
                        if Item._Type=='Frame' or Item._Type=='Canvas' or Item._Type=='Scroll' or Item._Type=='Popup':
                            Element_Type = 'Frame'
                        elif 'Canvas_' in Item._Type:
                            Element_Type = 'Item'
                        else:
                            Element_Type = 'Widget'
                        Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Item._ID}'", Keys=True)
                        Element_Data = Element_Data[0]
                        if Element_Data['Alignment']=='Percentage':
                            Fixture_Reverse = Item._Main.Locate_Reverse(0, 0, New_Left, New_Top)
                        else:
                            Fixture_Reverse = [0, 0, round(New_Left, 3), round(New_Top, 3)]
                        self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Left`={Fixture_Reverse[2]}, `Top`={Fixture_Reverse[3]} WHERE `ID`='{Item._ID}'")
                if self.Design.Configure.Current:
                    self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Drag_Start_Size(self, E, Element):
        try:
            self.Clear_Selection(Element)
            if not Element.Lock and not self.Dragging:
                self._Coord = {'X': E.x_root, 'Y': E.y_root}
                self.Element_Fixture = Element.Config_Get('Width', 'Height', 'Left', 'Top')
                Box = Element.Box()
                if self.Element_Fixture['Width'] > 0 and self.Element_Fixture['Height'] > 0 and Box[2] > 0 and Box[3] > 0:
                    self.Width_Ratio = Box[2] / self.Element_Fixture['Width']
                    self.Height_Ratio = Box[3] / self.Element_Fixture['Height']
                else:
                    self.Width_Ratio = 1.0
                    self.Height_Ratio = 1.0
                self.Resize_Direction = []
                Widget_Width_Px = Box[2]
                Widget_Height_Px = Box[3]
                if E.x < (Widget_Width_Px / 2):
                    self.Resize_Direction.append('Left')
                else:
                    self.Resize_Direction.append('Right')
                if E.y < (Widget_Height_Px / 2):
                    self.Resize_Direction.append('Top')
                else:
                    self.Resize_Direction.append('Bottom')
                if len(self.Element_Fixture) >= 4:
                    self.Dragging = True
                    self.Current = Element._ID
                    self.Window = getattr(self.Design, self.Parent)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
        self.Global['Loading'].Hide()

    def Drag_Size(self, E, Element):
        try:
            if not Element.Lock and self.Dragging and self.Current == Element._ID:
                Diff_X = (E.x_root - self._Coord['X']) / self.Width_Ratio
                Diff_Y = (E.y_root - self._Coord['Y']) / self.Height_Ratio
                New_Width = self.Element_Fixture['Width']
                New_Height = self.Element_Fixture['Height']
                New_Left = self.Element_Fixture['Left']
                New_Top = self.Element_Fixture['Top']
                if 'Left' in self.Resize_Direction:
                    Calculated_Left = self.Element_Fixture['Left'] + Diff_X
                    if self.Grid_Lock:
                        Calculated_Left = round(Calculated_Left / self.Grid_Width) * self.Grid_Width
                    Delta_Left = Calculated_Left - self.Element_Fixture['Left']
                    New_Left = Calculated_Left
                    New_Width = self.Element_Fixture['Width'] - Delta_Left
                elif 'Right' in self.Resize_Direction:
                    New_Width = self.Element_Fixture['Width'] + Diff_X
                    if self.Grid_Lock:
                        New_Width = round(New_Width / self.Grid_Width) * self.Grid_Width
                if 'Top' in self.Resize_Direction:
                    Calculated_Top = self.Element_Fixture['Top'] + Diff_Y
                    if self.Grid_Lock:
                        Calculated_Top = round(Calculated_Top / self.Grid_Height) * self.Grid_Height
                    Delta_Top = Calculated_Top - self.Element_Fixture['Top']
                    New_Top = Calculated_Top
                    New_Height = self.Element_Fixture['Height'] - Delta_Top
                elif 'Bottom' in self.Resize_Direction:
                    New_Height = self.Element_Fixture['Height'] + Diff_Y
                    if self.Grid_Lock:
                        New_Height = round(New_Height / self.Grid_Height) * self.Grid_Height
                Element.Size(Width=New_Width, Height=New_Height)
                Element.Position(Left=New_Left, Top=New_Top)
                if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                    Element_Type = 'Frame'
                elif 'Canvas_' in Element._Type:
                    Element_Type = 'Item'
                else:
                    Element_Type = 'Widget'
                Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                Element_Data = Element_Data[0]
                if Element_Data['Alignment'] == 'Percentage':
                    Fixture_Reverse = Element._Main.Locate_Reverse(New_Width, New_Height, New_Left, New_Top)
                else:
                    Fixture_Reverse = [round(New_Width, 3), round(New_Height, 3), round(New_Left, 3), round(New_Top, 3)]
                self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Width`={Fixture_Reverse[0]}, `Height`={Fixture_Reverse[1]}, `Left`={Fixture_Reverse[2]}, `Top`={Fixture_Reverse[3]} WHERE `ID`='{Element._ID}'")
                if self.Design.Configure.Current:
                    self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Show_Current(self, Element, Loading=True):
        try:
            if Loading and not self.Dragging:
                self.Clear_Selection(Element)
                self.Hide_Delete()
                self.Global['Loading'].Show()
                self.Global['GUI'].After(10, lambda: self.Show_Current(Element, Loading=False))
                self.Dragging = True
            else:
                self.Design.Configure.Hide_All()
                self.Design.Configure.Reset_All()
                ID_Tree = getattr(self, Element._ID)
                Parent = self.Tree.Parent(ID_Tree)
                self.Expand_Parent(Parent)
                self.Tree.Select(ID_Tree)
                Configure = getattr(self.Design.Configure, f'Configure_{Element._Type}')
                Configure.Load(Element._ID)
                self.Dragging = False
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Select_Current(self, Element):
        try:
            if len(self.Selected)==0:
                self.Hide_Delete()
                self.Design.Configure.Hide_All()
                self.Design.Configure.Reset_All()
            if Element not in self.Selected:
                self.Selected.append(Element)
                ID_Tree = getattr(self, Element._ID)
                Parent = self.Tree.Parent(ID_Tree)
                self.Expand_Parent(Parent)
                self.Tree.Select(ID_Tree, Keep=True)
            else:
                self.Selected.remove(Element)
                ID_Tree = getattr(self, Element._ID)
                self.Tree.Unselect(ID_Tree)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Clear_Selection(self, Element=None):
        try:
            self.Tree.Unselect()
            self.Selected = []
            if Element:
                self.Selected = [Element]
                ID_Tree = getattr(self, Element._ID)
                Parent = self.Tree.Parent(ID_Tree)
                self.Expand_Parent(Parent)
                self.Tree.Select(ID_Tree, Keep=True)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Expand_Parent(self, ID):
        try:
            self.Tree.Expand(ID)
            Name = self.Tree.Get(ID)[0]
            if Name and Name!=self.Parent:
                Parent = self.Tree.Parent(ID)
                self.Expand_Parent(Parent)
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Default(self, Parent='Root'):
        try:
            self.Parent = Parent
            self.Tree.Remove_All()
            setattr(self, self.Parent, self.Tree.Add(f" {self.Parent}", Value=[self.Parent, 'Frame', 0]))
            self.Load()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Reset_All(self):
        try:
            Widget = self.Tree.Get()
            if len(Widget)>0:
                if Widget[0]!=self.Current:
                    self.Current = Widget[0]
                    self.Design.Configure.Hide_All()
                    self.Design.Configure.Reset_All()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Get_Current(self, Loading=True):
        try:
            Widget = self.Tree.Get()
            if Widget and self.Current!=Widget[0]:
                if Loading:
                    self.Global['Loading'].Show()
                    self.Global['GUI'].After(10, lambda: self.Get_Current(Loading=False))
                else:
                    Widget = self.Tree.Get()
                    if len(Widget)>0:
                        if self.Current!=Widget[0]:
                            self.Current = Widget[0]
                            self.Hide_Delete()
                            self.Design.Configure.Hide_All()
                            self.Design.Configure.Reset_All()
                            if Widget[0]!=self.Parent:
                                Configure = getattr(self.Design.Configure, f'Configure_{Widget[1]}')
                                Configure.Load(Widget[0])
                                self.Selected = [Configure.Element]
                            else:
                                self.Delete_Image.Hide()
                                self.Copy_Image.Hide()
                                if not self.Copy_ID:
                                    self.Paste_Image.Hide()
                                self.Global['Loading'].Hide()
                        else:
                            self.Global['Loading'].Hide()
                    else:
                        self.Global['Loading'].Hide()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Get_Control_Current(self):
        try:
            Items = self.Tree.Get_Selected()
            if len(Items)>0:
                self.Selected = []
                for Item in Items:
                    Widget = Item['values']
                    if Widget[0]!=self.Parent:
                        self.Selected.append(getattr(self, f"{Widget[0]}_Widget"))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Show_Delete(self):
        try:
            self.Delete_Image.Set(self.Global['Image']('Close'))
            self.Delete_Image.Bind(Add=False, On_Click=lambda E: self.Hide_Delete())
            self.Copy_Image.Hide()
            self.Paste_Image.Hide()
            self.Delete_Confirm_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Hide_Delete(self):
        try:
            self.Delete_Confirm_Image.Hide()
            self.Delete_Image.Set(self.Global['Image']('Delete'))
            self.Delete_Image.Bind(Add=False, On_Click=lambda E: self.Show_Delete())
            self.Delete_Image.Show()
            self.Copy_Image.Show()
            if self.Copy_ID:
                self.Paste_Image.Show()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Copy(self):
        try:
            Temp_ID = self.Tree.Get_Selected()
            if len(Temp_ID)==1:
                ID = self.Tree.Current()
                if ID:
                    Element_ID = self.Tree.Get(ID)[0]
                    if Element_ID!=self.Parent:
                        self.Copy_ID = ID
                        self.Paste_Image.Show()
            else:
                self.Copy_ID = False
                self.Paste_Image.Hide()
                self.Global['Message'].Show('Error', 'Only Single Widget Copy Supported')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Paste(self):
        try:
            Root_ID = self.Tree.Current()
            if Root_ID:
                Root_Info = self.Tree.Get(Root_ID)
                while Root_Info[1]!='Frame' and Root_Info[1]!='Canvas' and Root_Info[1]!='Scroll' and Root_Info[1]!='Group':
                    Root_ID = self.Tree.Parent(Root_ID)
                    Root_Info = self.Tree.Get(Root_ID)
                if Root_Info[1]=='Frame' or Root_Info[1]=='Canvas' or Root_Info[1]=='Scroll' or Root_Info[1]=='Group':
                    Error = False
                    if Root_ID!=self.Copy_ID:
                        Parent = Root_ID
                        while self.Tree.Get(Parent)[0]!=self.Parent:
                            if Parent==self.Copy_ID:
                                Error = True
                                break
                            Parent = self.Tree.Parent(Parent)
                    else:
                        Root_ID = self.Tree.Parent(Root_ID)
                    if not Error:
                        self.Paste_All(self.Copy_ID, Root_ID)
                    else:
                        self.Global['Message'].Show('Error', 'Copy Not Possible Into Same Widget')
                else:
                    self.Global['Message'].Show('Error', 'Copy To Selected Widget Not Posiible')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Paste_All(self, ID=False, Root=False):
        try:
            if ID and Root:
                Root_ID = self.Tree.Get(Root)[0]
                Element_ID = self.Tree.Get(ID)[0]
                if len(self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Frame'
                elif len(self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                    Type = 'Item'
                else:
                    Type = 'Widget'
                Run = True
                if Type == 'Item':
                    if Root_ID!='Root':
                        Run = False
                        Widget = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Root_ID}'", Keys=True)
                        while Widget[0]['Type']=='Group' and Widget[0]['Root']!='Root':
                            Temp_Root_ID = Widget[0]['Root']
                            Widget = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root_ID}'", Keys=True)
                        if Widget[0]['Type']=='Canvas' or Widget[0]['Type']=='Scroll':
                            Run = True
                    else:
                        Run = False
                if Run:
                    self.Design.Database.Post(f"DELETE FROM `{Type}_Copy`")
                    self.Design.Database.Post(f"INSERT INTO `{Type}_Copy` SELECT * FROM `{Type}` WHERE `ID`='{Element_ID}'")
                    Frame_Data = self.Design.Database.Get(f"SELECT * FROM `{Type}_Copy` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    Match = re.match(r'^(.*?)(\d+)?$', Frame_Data['Name'])
                    Base_Name = Match.group(1)
                    Number = int(Match.group(2)) if Match.group(2) else 0
                    Name_Attempt = f"{Base_Name}{Number}" if Number > 0 else Base_Name
                    Exist = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Name_Attempt}' AND `Root`='{Root_ID}')")
                    while len(Exist) > 0:
                        Number += 1
                        Name_Attempt = f"{Base_Name}{Number}"
                        Exist = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE (`Name`='{Name_Attempt}' AND `Root`='{Root_ID}')")
                    Name = Name_Attempt
                    Random_Letter = ''.join(random.choices(string.ascii_letters, k=10))
                    New_ID = Random_Letter+self.Global['Custom'].MD5(Root_ID+Name+str(time.time()*1000000))
                    self.Design.Database.Post(f"UPDATE `{Type}_Copy` SET `ID`='{New_ID}',`Name`='{Name}',`Root`='{Root_ID}' WHERE `ID`='{Element_ID}'")
                    self.Design.Database.Post(f"INSERT INTO `{Type}` SELECT * FROM `{Type}_Copy` WHERE `ID`='{New_ID}'")
                    self.Design.Database.Post(f"DELETE FROM `{Type}_Copy` WHERE `ID`='{New_ID}'")
                    if os.path.exists(f"{self.Design.Project_Path}/Data/File/{Element_ID}"):
                        shutil.copy(f"{self.Design.Project_Path}/Data/File/{Element_ID}", f"{self.Design.Project_Path}/Data/File/{New_ID}")
                    if Type=='Frame':
                        Root_Level = self.Tree.Get(Root)[2]
                        self.Design.Database.Post(f"UPDATE `{Type}` SET `Level`='{Root_Level+1}' WHERE `ID`='{New_ID}'")
                    Widget = self.Design.Database.Get(f"SELECT * FROM `{Type}` WHERE `ID`='{New_ID}'", Keys=True)[0]
                    Stock = getattr(self.Design.Stock, f"Stock_{Widget['Type']}")
                    Stock.Create(Widget['ID'])
                    Level = 0
                    if Type=='Frame':
                        Level = Widget['Level']
                    Image = self.Global['Image'](Widget['Type'])
                    setattr(self, Widget['ID'], self.Tree.Add(Name=f" {Widget['Name']}", Parent=Root, Value=[Widget['ID'], Widget['Type'], Level], Path=Image))
                    for Each in self.Tree.Child(ID):
                        self.Paste_All(Each, getattr(self, Widget['ID']))
        except Exception as E:
            traceback.print_exc()
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Delete(self):
        try:
            ID = self.Tree.Selected()
            for Each in ID:
                self.Delete_All(Each)
            self.Hide_Delete()
            self.Design.Configure.Hide_All()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Delete_All(self, ID=False):
        try:
            if ID:
                if self.Copy_ID==ID:
                    self.Copy_ID = False
                    self.Paste_Image.Hide()
                for Each in self.Tree.Child(ID):
                    self.Delete_All(Each)
                Element_ID = self.Tree.Get(ID)[0]
                if Element_ID!=self.Parent:
                    if len(self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    elif len(self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True))>0:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Item` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    else:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Widget` WHERE `ID`='{Element_ID}'", Keys=True)[0]
                    Temp_Root = Frame_Data['Root']
                    Root = Temp_Root
                    while Temp_Root!=self.Parent:
                        Frame_Data = self.Design.Database.Get(f"SELECT * FROM `Frame` WHERE `ID`='{Temp_Root}'", Keys=True)
                        if len(Frame_Data)==1:
                            Frame_Data = Frame_Data[0]
                            Temp_Root = Frame_Data['Root']
                            Root = Temp_Root+'.'+Root
                        else:
                            break
                    if os.path.exists(f"{self.Design.Project_Path}/Data/File/{Element_ID}"):
                        os.remove(f"{self.Design.Project_Path}/Data/File/{Element_ID}")
                    Root = self.Global['Custom'].Get_Attr_Class(self.Design, Root)
                    Element = getattr(Root, Element_ID)
                    Element.Delete()
                    self.Tree.Remove(ID)
                    self.Design.Database.Post(f"DELETE FROM `Frame` WHERE `ID`='{Element_ID}'")
                    self.Design.Database.Post(f"DELETE FROM `Item` WHERE `ID`='{Element_ID}'")
                    self.Design.Database.Post(f"DELETE FROM `Widget` WHERE `ID`='{Element_ID}'")
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
    
    def Export(self):
        try:
            if not os.path.exists(f"{self.Design.Project_Path}/Structure"):
                os.mkdir(f"{self.Design.Project_Path}/Structure")
            self.Tree.Export(Path=f"{self.Design.Project_Path}/Structure/{self.Parent}.txt")
            self.Global['Message'].Show('Success', 'Structure Exported To Project Folder')
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Grid(self):
        try:
            if self.Grid_Lock:
                self.Grid_Lock = False
                self.Grid_Image.Set(self.Global['Image']('Grid_Unlock'))
            else:
                self.Grid_Lock = True
                self.Grid_Image.Set(self.Global['Image']('Grid_Lock'))
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Align_Left(self):
        try:
            if len(self.Selected)>0:
                Reference_Main = self.Selected[0]._Main
                Same_Parent = True
                for Element in self.Selected:
                    if Element._Main != Reference_Main:
                        Same_Parent = False
                        self.Global['Message'].Show('Error', 'Align Same Container Elements Only')
                        break
                if Same_Parent:
                    Left_Values = []
                    for Element in self.Selected:
                        Config = Element.Config_Get('Left')
                        Left_Values.append(Config['Left'])
                    Target_Left = min(Left_Values)
                    for Element in self.Selected:
                        if not Element.Lock:
                            Element.Position(Left=Target_Left)
                            if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                                Element_Type = 'Frame'
                            elif 'Canvas_' in Element._Type:
                                Element_Type = 'Item'
                            else:
                                Element_Type = 'Widget'
                            Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                            Element_Data = Element_Data[0]
                            if Element_Data['Alignment']=='Percentage':
                                Config = Element.Config_Get('Width', 'Height', 'Top')
                                Fixture_Reverse = Element._Main.Locate_Reverse(Config['Width'], Config['Height'], Target_Left, Config['Top'])
                                New_Left = Fixture_Reverse[2]
                            else:
                                New_Left = round(Target_Left, 3)
                            self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Left`={New_Left} WHERE `ID`='{Element._ID}'")
                            if self.Design.Configure.Current:
                                self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Align_Top(self):
        try:
            if len(self.Selected)>0:
                Reference_Main = self.Selected[0]._Main
                Same_Parent = True
                for Element in self.Selected:
                    if Element._Main != Reference_Main:
                        Same_Parent = False
                        self.Global['Message'].Show('Error', 'Align Same Container Elements Only')
                        break
                if Same_Parent:
                    Top_Values = []
                    for Element in self.Selected:
                        Config = Element.Config_Get('Top')
                        Top_Values.append(Config['Top'])
                    Target_Top = min(Top_Values)
                    for Element in self.Selected:
                        if not Element.Lock:
                            Element.Position(Top=Target_Top)
                            if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                                Element_Type = 'Frame'
                            elif 'Canvas_' in Element._Type:
                                Element_Type = 'Item'
                            else:
                                Element_Type = 'Widget'
                            Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                            Element_Data = Element_Data[0]
                            if Element_Data['Alignment']=='Percentage':
                                Config = Element.Config_Get('Width', 'Height', 'Left')
                                Fixture_Reverse = Element._Main.Locate_Reverse(Config['Width'], Config['Height'], Config['Left'], Target_Top)
                                New_Top = Fixture_Reverse[3]
                            else:
                                New_Top = round(Target_Top, 3)
                            self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Top`={New_Top} WHERE `ID`='{Element._ID}'")
                            if self.Design.Configure.Current:
                                self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Align_Right(self):
        try:
            if len(self.Selected)>0:
                Reference_Main = self.Selected[0]._Main
                Same_Parent = True
                for Element in self.Selected:
                    if Element._Main != Reference_Main:
                        Same_Parent = False
                        self.Global['Message'].Show('Error', 'Align Same Container Elements Only')
                        break
                if Same_Parent:
                    Right_Values = []
                    for Element in self.Selected:
                        Config = Element.Config_Get('Left', 'Width')
                        Right_Values.append(Config['Left'] + Config['Width'])
                    Target_Right = max(Right_Values)
                    for Element in self.Selected:
                        if not Element.Lock:
                            Config_Current = Element.Config_Get('Width')
                            Target_Left = Target_Right - Config_Current['Width']
                            Element.Position(Left=Target_Left)
                            if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                                Element_Type = 'Frame'
                            elif 'Canvas_' in Element._Type:
                                Element_Type = 'Item'
                            else:
                                Element_Type = 'Widget'
                            Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                            Element_Data = Element_Data[0]
                            if Element_Data['Alignment']=='Percentage':
                                Config = Element.Config_Get('Width', 'Height', 'Top')
                                Fixture_Reverse = Element._Main.Locate_Reverse(Config['Width'], Config['Height'], Target_Left, Config['Top'])
                                New_Left = Fixture_Reverse[2]
                            else:
                                New_Left = round(Target_Left, 3)
                            self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Left`={New_Left} WHERE `ID`='{Element._ID}'")
                            if self.Design.Configure.Current:
                                self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))

    def Align_Bottom(self):
        try:
            if len(self.Selected)>0:
                Reference_Main = self.Selected[0]._Main
                Same_Parent = True
                for Element in self.Selected:
                    if Element._Main != Reference_Main:
                        Same_Parent = False
                        self.Global['Message'].Show('Error', 'Align Same Container Elements Only')
                        break
                if Same_Parent:
                    Bottom_Values = []
                    for Element in self.Selected:
                        Config = Element.Config_Get('Top', 'Height')
                        Bottom_Values.append(Config['Top'] + Config['Height'])
                    Target_Bottom = max(Bottom_Values)
                    for Element in self.Selected:
                        if not Element.Lock:
                            Config_Current = Element.Config_Get('Height')
                            Target_Top = Target_Bottom - Config_Current['Height']
                            Element.Position(Top=Target_Top)
                            if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                                Element_Type = 'Frame'
                            elif 'Canvas_' in Element._Type:
                                Element_Type = 'Item'
                            else:
                                Element_Type = 'Widget'
                            Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                            Element_Data = Element_Data[0]
                            if Element_Data['Alignment']=='Percentage':
                                Config = Element.Config_Get('Width', 'Height', 'Left')
                                Fixture_Reverse = Element._Main.Locate_Reverse(Config['Width'], Config['Height'], Config['Left'], Target_Top)
                                New_Top = Fixture_Reverse[3]
                            else:
                                New_Top = round(Target_Top, 3)
                            self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Top`={New_Top} WHERE `ID`='{Element._ID}'")
                            if self.Design.Configure.Current:
                                self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))
            
    def Align_Center(self):
        try:
            if len(self.Selected)>0:
                Reference_Main = self.Selected[0]._Main
                Same_Parent = True
                for Element in self.Selected:
                    if Element._Main != Reference_Main:
                        Same_Parent = False
                        self.Global['Message'].Show('Error', 'Align Same Container Elements Only')
                        break
                if Same_Parent:
                    Left_Values = []
                    Right_Values = []
                    for Element in self.Selected:
                        Config = Element.Config_Get('Left', 'Width')
                        Left_Values.append(Config['Left'])
                        Right_Values.append(Config['Left'] + Config['Width'])
                    Target_Center = (min(Left_Values) + max(Right_Values)) / 2
                    for Element in self.Selected:
                        if not Element.Lock:
                            Config_Current = Element.Config_Get('Width')
                            Target_Left = Target_Center - (Config_Current['Width'] / 2)
                            Element.Position(Left=Target_Left)
                            if Element._Type=='Frame' or Element._Type=='Canvas' or Element._Type=='Scroll' or Element._Type=='Popup':
                                Element_Type = 'Frame'
                            elif 'Canvas_' in Element._Type:
                                Element_Type = 'Item'
                            else:
                                Element_Type = 'Widget'
                            Element_Data = self.Design.Database.Get(f"SELECT * FROM `{Element_Type}` WHERE `ID`='{Element._ID}'", Keys=True)
                            Element_Data = Element_Data[0]
                            if Element_Data['Alignment']=='Percentage':
                                Config = Element.Config_Get('Width', 'Height', 'Top')
                                Fixture_Reverse = Element._Main.Locate_Reverse(Config['Width'], Config['Height'], Target_Left, Config['Top'])
                                New_Left = Fixture_Reverse[2]
                            else:
                                New_Left = round(Target_Left, 3)
                            self.Design.Database.Post(f"UPDATE `{Element_Type}` SET `Left`={New_Left} WHERE `ID`='{Element._ID}'")
                            if self.Design.Configure.Current:
                                self.Design.Configure.Current.Movement_Update()
        except Exception as E:
            self.Global['Error'](__class__.__name__+" -> "+inspect.currentframe().f_code.co_name+" -> "+str(E))